@@ -10,13 +10,13 @@ impl ChunkedBuilder<bool, BooleanType> for BooleanChunkedBuilder {
     /// Appends a value of type `T` into the builder
     #[inline]
     fn append_value(&mut self, v: bool) {
-        self.array_builder.push(Some(v));
+        self.array_builder.push_value(v);
     }
 
     /// Appends a null slot into the builder
     #[inline]
     fn append_null(&mut self) {
-        self.array_builder.push(None);
+        self.array_builder.push_null();
     }
 
     fn finish(mut self) -> BooleanChunked {
