@@ -55,8 +55,8 @@ use crate::chunked_array::object::PolarsObjectSafe;
 use crate::prelude::*;
 use crate::utils::Wrap;
 
-pub struct Nested;
-pub struct Flat;
+pub struct TrueT;
+pub struct FalseT;
 
 /// # Safety
 ///
@@ -68,7 +68,8 @@ pub unsafe trait PolarsDataType: Send + Sync + Sized {
         ValueT<'a> = Self::Physical<'a>,
         ZeroableValueT<'a> = Self::ZeroablePhysical<'a>,
     >;
-    type Structure;
+    type IsNested;
+    type HasViews;
 
     fn get_dtype() -> DataType
     where
@@ -81,7 +82,8 @@ where
         Physical<'a> = Self::Native,
         ZeroablePhysical<'a> = Self::Native,
         Array = PrimitiveArray<Self::Native>,
-        Structure = Flat,
+        IsNested = FalseT,
+        HasViews = FalseT,
     >,
 {
     type Native: NumericNative;
@@ -99,7 +101,8 @@ macro_rules! impl_polars_num_datatype {
             type Physical<'a> = $physical;
             type ZeroablePhysical<'a> = $physical;
             type Array = PrimitiveArray<$physical>;
-            type Structure = Flat;
+            type IsNested = FalseT;
+            type HasViews = FalseT;
 
             #[inline]
             fn get_dtype() -> DataType {
@@ -115,8 +118,8 @@ macro_rules! impl_polars_num_datatype {
     };
 }
 
-macro_rules! impl_polars_datatype {
-    ($ca:ident, $variant:ident, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty) => {
+macro_rules! impl_polars_datatype_pass_dtype {
+    ($ca:ident, $dtype:expr, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty, $has_views:ident) => {
         #[derive(Clone, Copy)]
         pub struct $ca {}
 
@@ -124,15 +127,43 @@ macro_rules! impl_polars_datatype {
             type Physical<$lt> = $phys;
             type ZeroablePhysical<$lt> = $zerophys;
             type Array = $arr;
-            type Structure = Flat;
+            type IsNested = FalseT;
+            type HasViews = $has_views;
 
             #[inline]
             fn get_dtype() -> DataType {
-                DataType::$variant
+                $dtype
             }
         }
     };
 }
+macro_rules! impl_polars_binview_datatype {
+    ($ca:ident, $variant:ident, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty) => {
+        impl_polars_datatype_pass_dtype!(
+            $ca,
+            DataType::$variant,
+            $arr,
+            $lt,
+            $phys,
+            $zerophys,
+            TrueT
+        );
+    };
+}
+
+macro_rules! impl_polars_datatype {
+    ($ca:ident, $variant:ident, $arr:ty, $lt:lifetime, $phys:ty, $zerophys:ty) => {
+        impl_polars_datatype_pass_dtype!(
+            $ca,
+            DataType::$variant,
+            $arr,
+            $lt,
+            $phys,
+            $zerophys,
+            FalseT
+        );
+    };
+}
 
 impl_polars_num_datatype!(PolarsIntegerType, UInt8Type, UInt8, u8);
 impl_polars_num_datatype!(PolarsIntegerType, UInt16Type, UInt16, u16);
@@ -145,24 +176,26 @@ impl_polars_num_datatype!(PolarsIntegerType, Int64Type, Int64, i64);
 impl_polars_num_datatype!(PolarsFloatType, Float32Type, Float32, f32);
 impl_polars_num_datatype!(PolarsFloatType, Float64Type, Float64, f64);
 impl_polars_datatype!(DateType, Date, PrimitiveArray<i32>, 'a, i32, i32);
-#[cfg(feature = "dtype-decimal")]
-impl_polars_datatype!(DecimalType, Unknown, PrimitiveArray<i128>, 'a, i128, i128);
-impl_polars_datatype!(DatetimeType, Unknown, PrimitiveArray<i64>, 'a, i64, i64);
-impl_polars_datatype!(DurationType, Unknown, PrimitiveArray<i64>, 'a, i64, i64);
-impl_polars_datatype!(CategoricalType, Unknown, PrimitiveArray<u32>, 'a, u32, u32);
 impl_polars_datatype!(TimeType, Time, PrimitiveArray<i64>, 'a, i64, i64);
-impl_polars_datatype!(StringType, String, Utf8ViewArray, 'a, &'a str, Option<&'a str>);
-impl_polars_datatype!(BinaryType, Binary, BinaryViewArray, 'a, &'a [u8], Option<&'a [u8]>);
+impl_polars_binview_datatype!(StringType, String, Utf8ViewArray, 'a, &'a str, Option<&'a str>);
+impl_polars_binview_datatype!(BinaryType, Binary, BinaryViewArray, 'a, &'a [u8], Option<&'a [u8]>);
 impl_polars_datatype!(BinaryOffsetType, BinaryOffset, BinaryArray<i64>, 'a, &'a [u8], Option<&'a [u8]>);
 impl_polars_datatype!(BooleanType, Boolean, BooleanArray, 'a, bool, bool);
 
+#[cfg(feature = "dtype-decimal")]
+impl_polars_datatype_pass_dtype!(DecimalType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i128>, 'a, i128, i128, FalseT);
+impl_polars_datatype_pass_dtype!(DatetimeType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i64>, 'a, i64, i64, FalseT);
+impl_polars_datatype_pass_dtype!(DurationType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<i64>, 'a, i64, i64, FalseT);
+impl_polars_datatype_pass_dtype!(CategoricalType, DataType::Unknown(UnknownKind::Any), PrimitiveArray<u32>, 'a, u32, u32, FalseT);
+
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct ListType {}
 unsafe impl PolarsDataType for ListType {
     type Physical<'a> = Box<dyn Array>;
     type ZeroablePhysical<'a> = Option<Box<dyn Array>>;
     type Array = ListArray<i64>;
-    type Structure = Nested;
+    type IsNested = TrueT;
+    type HasViews = FalseT;
 
     fn get_dtype() -> DataType {
         // Null as we cannot know anything without self.
@@ -177,7 +210,8 @@ unsafe impl PolarsDataType for FixedSizeListType {
     type Physical<'a> = Box<dyn Array>;
     type ZeroablePhysical<'a> = Option<Box<dyn Array>>;
     type Array = FixedSizeListArray;
-    type Structure = Nested;
+    type IsNested = TrueT;
+    type HasViews = FalseT;
 
     fn get_dtype() -> DataType {
         // Null as we cannot know anything without self.
@@ -191,7 +225,8 @@ unsafe impl PolarsDataType for Int128Type {
     type Physical<'a> = i128;
     type ZeroablePhysical<'a> = i128;
     type Array = PrimitiveArray<i128>;
-    type Structure = Flat;
+    type IsNested = FalseT;
+    type HasViews = FalseT;
 
     fn get_dtype() -> DataType {
         // Scale is not None to allow for get_any_value() to work.
@@ -211,7 +246,8 @@ unsafe impl<T: PolarsObject> PolarsDataType for ObjectType<T> {
     type Physical<'a> = &'a T;
     type ZeroablePhysical<'a> = Option<&'a T>;
     type Array = ObjectArray<T>;
-    type Structure = Nested;
+    type IsNested = TrueT;
+    type HasViews = FalseT;
 
     fn get_dtype() -> DataType {
         DataType::Object(T::type_name(), None)
