@@ -1,13 +1,11 @@
 use std::fmt::{Debug, Display, Formatter};
 use std::hash::Hash;
 
-use ahash::RandomState;
 use num_traits::NumCast;
-use polars_utils::hashing::{BytesHash, DirtyHash};
+use polars_utils::hashing::DirtyHash;
 use rayon::prelude::*;
 
 use self::hashing::*;
-use crate::hashing::get_null_hash_value;
 use crate::prelude::*;
 use crate::utils::{_set_partition_size, accumulate_dataframes_vertical};
 use crate::POOL;
@@ -22,37 +20,9 @@ mod proxy;
 pub use into_groups::*;
 pub use proxy::*;
 
-#[cfg(feature = "dtype-struct")]
-use crate::prelude::sort::arg_sort_multiple::encode_rows_vertical;
-
-// This will remove the sorted flag on signed integers
-fn prepare_dataframe_unsorted(by: &[Series]) -> DataFrame {
-    let columns = by
-        .iter()
-        .map(|s| match s.dtype() {
-            #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                s.cast(&DataType::UInt32).unwrap()
-            },
-            _ => {
-                if s.dtype().to_physical().is_numeric() {
-                    let s = s.to_physical_repr();
-
-                    if s.dtype().is_float() {
-                        s.into_owned().into_series()
-                    } else if s.bit_repr_is_large() {
-                        s.bit_repr_large().into_series()
-                    } else {
-                        s.bit_repr_small().into_series()
-                    }
-                } else {
-                    s.clone()
-                }
-            },
-        })
-        .collect();
-    unsafe { DataFrame::new_no_checks(columns) }
-}
+use crate::prelude::sort::arg_sort_multiple::{
+    encode_rows_unordered, encode_rows_vertical_par_unordered,
+};
 
 impl DataFrame {
     pub fn group_by_with_series(
@@ -82,25 +52,47 @@ impl DataFrame {
             }
         };
 
-        let n_partitions = _set_partition_size();
-
         let groups = if by.len() == 1 {
             let series = &by[0];
             series.group_tuples(multithreaded, sorted)
-        } else {
-            #[cfg(feature = "dtype-struct")]
+        } else if by.iter().any(|s| s.dtype().is_object()) {
+            #[cfg(feature = "object")]
             {
-                if by.iter().any(|s| matches!(s.dtype(), DataType::Struct(_))) {
-                    let rows = encode_rows_vertical(&by)?;
-                    let groups = rows.group_tuples(multithreaded, sorted)?;
-                    return Ok(GroupBy::new(self, by, groups, None));
-                }
+                let mut df = DataFrame::new(by.clone()).unwrap();
+                let n = df.height();
+                let rows = df.to_av_rows();
+                let iter = (0..n).map(|i| rows.get(i));
+                Ok(group_by(iter, sorted))
             }
-            let keys_df = prepare_dataframe_unsorted(&by);
-            if multithreaded {
-                group_by_threaded_multiple_keys_flat(keys_df, n_partitions, sorted)
+            #[cfg(not(feature = "object"))]
+            {
+                unreachable!()
+            }
+        } else {
+            // Skip null dtype.
+            let by = by
+                .iter()
+                .filter(|s| !s.dtype().is_null())
+                .cloned()
+                .collect::<Vec<_>>();
+            if by.is_empty() {
+                let groups = if self.height() == 0 {
+                    vec![]
+                } else {
+                    vec![[0, self.height() as IdxSize]]
+                };
+                Ok(GroupsProxy::Slice {
+                    groups,
+                    rolling: false,
+                })
             } else {
-                group_by_multiple_keys(keys_df, sorted)
+                let rows = if multithreaded {
+                    encode_rows_vertical_par_unordered(&by)
+                } else {
+                    encode_rows_unordered(&by)
+                }?
+                .into_series();
+                rows.group_tuples(multithreaded, sorted)
             }
         };
         Ok(GroupBy::new(self, by, groups?, None))
@@ -1092,7 +1084,7 @@ mod test {
         // Use of deprecated `sum()` for testing purposes
         #[allow(deprecated)]
         let res = df.group_by(["flt"]).unwrap().sum().unwrap();
-        let res = res.sort(["flt"], false, false).unwrap();
+        let res = res.sort(["flt"], SortMultipleOptions::default()).unwrap();
         assert_eq!(
             Vec::from(res.column("val_sum").unwrap().i32().unwrap()),
             &[Some(2), Some(2), Some(1)]
