@@ -106,6 +106,11 @@ impl PrivateSeries for NullChunked {
         })
     }
 
+    #[cfg(feature = "algorithm_group_by")]
+    unsafe fn agg_list(&self, groups: &GroupsProxy) -> Series {
+        AggList::agg_list(self, groups)
+    }
+
     fn _get_flags(&self) -> Settings {
         Settings::empty()
     }
@@ -183,6 +188,10 @@ impl SeriesTrait for NullChunked {
         NullChunked::new(self.name.clone(), self.len()).into_series()
     }
 
+    fn drop_nulls(&self) -> Series {
+        NullChunked::new(self.name.clone(), 0).into_series()
+    }
+
     fn cast(&self, data_type: &DataType) -> PolarsResult<Series> {
         Ok(Series::full_null(self.name.as_ref(), self.len(), data_type))
     }
