@@ -2,21 +2,6 @@ use crate::prelude::*;
 use crate::series::unstable::UnstableSeries;
 use crate::series::IsSorted;
 
-/// Transform to physical type and coerce similar sized integer to a bit representation
-/// to reduce compiler bloat
-pub fn _to_physical_and_bit_repr(s: &[Series]) -> Vec<Series> {
-    s.iter()
-        .map(|s| {
-            let physical = s.to_physical_repr();
-            match physical.dtype() {
-                DataType::Int64 => physical.bit_repr_large().into_series(),
-                DataType::Int32 => physical.bit_repr_small().into_series(),
-                _ => physical.into_owned(),
-            }
-        })
-        .collect()
-}
-
 /// A utility that allocates an [`UnstableSeries`]. The applied function can then use that
 /// series container to save heap allocations and swap arrow arrays.
 pub fn with_unstable_series<F, T>(dtype: &DataType, f: F) -> T
