@@ -1,3 +1,5 @@
+use num_traits::Signed;
+
 use super::*;
 
 /// Given two data types, determine the data type that both types can safely be cast to.
@@ -195,9 +197,9 @@ pub fn get_supertype(l: &DataType, r: &DataType) -> Option<DataType> {
             (Time, Float64) => Some(Float64),
 
             // every known type can be casted to a string except binary
-            (dt, String) if dt != &DataType::Unknown && dt != &DataType::Binary => Some(String),
+            (dt, String) if !matches!(dt, DataType::Unknown(UnknownKind::Any)) && dt != &DataType::Binary => Some(String),
 
-            (dt, String) if dt != &DataType::Unknown => Some(String),
+            (dt, String) if !matches!(dt, DataType::Unknown(UnknownKind::Any)) => Some(String),
 
             (dt, Null) => Some(dt.clone()),
 
@@ -253,7 +255,43 @@ pub fn get_supertype(l: &DataType, r: &DataType) -> Option<DataType> {
                 let st = get_supertype(inner_left, inner_right)?;
                 Some(DataType::List(Box::new(st)))
             }
-            (_, Unknown) => Some(Unknown),
+            #[cfg(feature = "dtype-struct")]
+            (Struct(inner), right @ Unknown(UnknownKind::Float | UnknownKind::Int(_))) => {
+                match inner.first() {
+                    Some(inner) => get_supertype(&inner.dtype, right),
+                    None => None
+                }
+            },
+            (dt, Unknown(kind)) => {
+                match kind {
+                    UnknownKind::Float | UnknownKind::Int(_) if dt.is_float() | dt.is_string() => Some(dt.clone()),
+                    UnknownKind::Float if dt.is_numeric() => Some(Unknown(UnknownKind::Float)),
+                    UnknownKind::Str if dt.is_string() | dt.is_enum() => Some(dt.clone()),
+                    #[cfg(feature = "dtype-categorical")]
+                    UnknownKind::Str if dt.is_categorical()  => {
+                        let Categorical(_, ord) = dt else { unreachable!()};
+                        Some(Categorical(None, *ord))
+                    },
+                    dynam if dt.is_null() => Some(Unknown(*dynam)),
+                    UnknownKind::Int(v) if dt.is_numeric() => {
+                        let smallest_fitting_dtype = if dt.is_unsigned_integer() && v.is_positive() {
+                            materialize_dyn_int_pos(*v).dtype()
+                        } else {
+                            materialize_smallest_dyn_int(*v).dtype()
+                        };
+                        match dt {
+                            UInt64 if smallest_fitting_dtype.is_signed_integer() => {
+                                // Ensure we don't cast to float when dealing with dynamic literals
+                                Some(Int64)
+                            },
+                            _ => {
+                                get_supertype(dt, &smallest_fitting_dtype)
+                            }
+                        }
+                    }
+                    _ => Some(Unknown(UnknownKind::Any))
+                }
+            },
             #[cfg(feature = "dtype-struct")]
             (Struct(fields_a), Struct(fields_b)) => {
                 super_type_structs(fields_a, fields_b)
@@ -341,3 +379,54 @@ fn super_type_structs(fields_a: &[Field], fields_b: &[Field]) -> Option<DataType
         Some(DataType::Struct(new_fields))
     }
 }
+
+pub fn materialize_dyn_int(v: i128) -> AnyValue<'static> {
+    // Try to get the "smallest" fitting value.
+    // TODO! next breaking go to true smallest.
+    match i32::try_from(v).ok() {
+        Some(v) => AnyValue::Int32(v),
+        None => match i64::try_from(v).ok() {
+            Some(v) => AnyValue::Int64(v),
+            None => match u64::try_from(v).ok() {
+                Some(v) => AnyValue::UInt64(v),
+                None => AnyValue::Null,
+            },
+        },
+    }
+}
+fn materialize_dyn_int_pos(v: i128) -> AnyValue<'static> {
+    // Try to get the "smallest" fitting value.
+    // TODO! next breaking go to true smallest.
+    match u8::try_from(v).ok() {
+        Some(v) => AnyValue::UInt8(v),
+        None => match u16::try_from(v).ok() {
+            Some(v) => AnyValue::UInt16(v),
+            None => match u32::try_from(v).ok() {
+                Some(v) => AnyValue::UInt32(v),
+                None => match u64::try_from(v).ok() {
+                    Some(v) => AnyValue::UInt64(v),
+                    None => AnyValue::Null,
+                },
+            },
+        },
+    }
+}
+
+fn materialize_smallest_dyn_int(v: i128) -> AnyValue<'static> {
+    match i8::try_from(v).ok() {
+        Some(v) => AnyValue::Int8(v),
+        None => match i16::try_from(v).ok() {
+            Some(v) => AnyValue::Int16(v),
+            None => match i32::try_from(v).ok() {
+                Some(v) => AnyValue::Int32(v),
+                None => match i64::try_from(v).ok() {
+                    Some(v) => AnyValue::Int64(v),
+                    None => match u64::try_from(v).ok() {
+                        Some(v) => AnyValue::UInt64(v),
+                        None => AnyValue::Null,
+                    },
+                },
+            },
+        },
+    }
+}
