@@ -1,8 +1,12 @@
+#[cfg(any(feature = "ipc_streaming", feature = "parquet"))]
+use std::borrow::Cow;
 use std::io::Read;
 use std::path::{Path, PathBuf};
 
 use once_cell::sync::Lazy;
 use polars_core::prelude::*;
+#[cfg(any(feature = "ipc_streaming", feature = "parquet"))]
+use polars_core::utils::{accumulate_dataframes_vertical_unchecked, split_df_as_ref};
 use regex::{Regex, RegexBuilder};
 
 use crate::mmap::{MmapBytesReader, ReaderBytes};
@@ -168,6 +172,10 @@ pub static FLOAT_RE: Lazy<Regex> = Lazy::new(|| {
     Regex::new(r"^[-+]?((\d*\.\d+)([eE][-+]?\d+)?|inf|NaN|(\d+)[eE][-+]?\d+|\d+\.)$").unwrap()
 });
 
+pub static FLOAT_RE_DECIMAL: Lazy<Regex> = Lazy::new(|| {
+    Regex::new(r"^[-+]?((\d*,\d+)([eE][-+]?\d+)?|inf|NaN|(\d+)[eE][-+]?\d+|\d+,)$").unwrap()
+});
+
 pub static INTEGER_RE: Lazy<Regex> = Lazy::new(|| Regex::new(r"^-?(\d+)$").unwrap());
 
 pub static BOOLEAN_RE: Lazy<Regex> = Lazy::new(|| {
@@ -256,11 +264,77 @@ pub fn check_projected_schema(
     check_projected_schema_impl(a, b, projected_names, msg)
 }
 
+/// Split DataFrame into chunks in preparation for writing. The chunks have a
+/// maximum number of rows per chunk to ensure reasonable memory efficiency when
+/// reading the resulting file, and a minimum size per chunk to ensure
+/// reasonable performance when writing.
+#[cfg(any(feature = "ipc_streaming", feature = "parquet"))]
+pub(crate) fn chunk_df_for_writing(
+    df: &mut DataFrame,
+    row_group_size: usize,
+) -> PolarsResult<Cow<DataFrame>> {
+    // ensures all chunks are aligned.
+    df.align_chunks();
+
+    let n_splits = df.height() / row_group_size;
+    let result = if n_splits > 0 {
+        Cow::Owned(accumulate_dataframes_vertical_unchecked(
+            split_df_as_ref(df, n_splits, false)?
+                .into_iter()
+                .map(|mut df| {
+                    // If the chunks are small enough, writing many small chunks
+                    // leads to slow writing performance, so in that case we
+                    // merge them.
+                    let n_chunks = df.n_chunks();
+                    if n_chunks > 1 && (df.estimated_size() / n_chunks < 128 * 1024) {
+                        df.as_single_chunk_par();
+                    }
+                    df
+                }),
+        ))
+    } else {
+        Cow::Borrowed(df)
+    };
+    Ok(result)
+}
+
+static CLOUD_URL: Lazy<Regex> =
+    Lazy::new(|| Regex::new(r"^(s3a?|gs|gcs|file|abfss?|azure|az|adl|https?)://").unwrap());
+
+/// Check if the path is a cloud url.
+pub fn is_cloud_url<P: AsRef<Path>>(p: P) -> bool {
+    match p.as_ref().as_os_str().to_str() {
+        Some(s) => CLOUD_URL.is_match(s),
+        _ => false,
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::path::PathBuf;
 
-    use super::resolve_homedir;
+    use super::{resolve_homedir, FLOAT_RE};
+
+    #[test]
+    fn test_float_parse() {
+        assert!(FLOAT_RE.is_match("0.1"));
+        assert!(FLOAT_RE.is_match("3.0"));
+        assert!(FLOAT_RE.is_match("3.00001"));
+        assert!(FLOAT_RE.is_match("-9.9990e-003"));
+        assert!(FLOAT_RE.is_match("9.9990e+003"));
+        assert!(FLOAT_RE.is_match("9.9990E+003"));
+        assert!(FLOAT_RE.is_match("9.9990E+003"));
+        assert!(FLOAT_RE.is_match(".5"));
+        assert!(FLOAT_RE.is_match("2.5E-10"));
+        assert!(FLOAT_RE.is_match("2.5e10"));
+        assert!(FLOAT_RE.is_match("NaN"));
+        assert!(FLOAT_RE.is_match("-NaN"));
+        assert!(FLOAT_RE.is_match("-inf"));
+        assert!(FLOAT_RE.is_match("inf"));
+        assert!(FLOAT_RE.is_match("-7e-05"));
+        assert!(FLOAT_RE.is_match("7e-05"));
+        assert!(FLOAT_RE.is_match("+7e+05"));
+    }
 
     #[cfg(not(target_os = "windows"))]
     #[test]
