@@ -4,19 +4,12 @@ use crate::physical_plan::planner::create_physical_expr;
 use crate::prelude::*;
 
 #[cfg(feature = "pivot")]
-pub(crate) fn prepare_eval_expr(mut expr: Expr) -> Expr {
-    expr.mutate().apply(|e| match e {
-        Expr::Column(name) => {
-            *name = Arc::from("");
-            true
-        },
-        Expr::Nth(_) => {
-            *e = Expr::Column(Arc::from(""));
-            true
-        },
-        _ => true,
-    });
-    expr
+pub(crate) fn prepare_eval_expr(expr: Expr) -> Expr {
+    expr.map_expr(|e| match e {
+        Expr::Column(_) => Expr::Column(Arc::from("")),
+        Expr::Nth(_) => Expr::Column(Arc::from("")),
+        e => e,
+    })
 }
 
 pub(crate) fn prepare_expression_for_context(
@@ -37,9 +30,9 @@ pub(crate) fn prepare_expression_for_context(
         .without_optimizations()
         .with_simplify_expr(true)
         .select([expr.clone()]);
-    let optimized = lf.optimize(&mut lp_arena, &mut expr_arena).unwrap();
+    let optimized = lf.optimize(&mut lp_arena, &mut expr_arena)?;
     let lp = lp_arena.get(optimized);
     let aexpr = lp.get_exprs().pop().unwrap();
 
-    create_physical_expr(aexpr, ctxt, &expr_arena, None, &mut Default::default())
+    create_physical_expr(&aexpr, ctxt, &expr_arena, None, &mut Default::default())
 }
