@@ -1,6 +1,3 @@
-use std::sync::Arc;
-
-use polars_core::frame::group_by::GroupsProxy;
 use polars_core::prelude::*;
 use polars_core::POOL;
 #[cfg(feature = "round_series")]
@@ -76,6 +73,12 @@ pub fn apply_operator(left: &Series, right: &Series, op: Operator) -> PolarsResu
         },
         Operator::And => left.bitand(right),
         Operator::Or => left.bitor(right),
+        Operator::LogicalOr => left
+            .cast(&DataType::Boolean)?
+            .bitor(&right.cast(&DataType::Boolean)?),
+        Operator::LogicalAnd => left
+            .cast(&DataType::Boolean)?
+            .bitand(&right.cast(&DataType::Boolean)?),
         Operator::Xor => left.bitxor(right),
         Operator::Modulus => Ok(left % right),
         Operator::EqValidity => left.equal_missing(right).map(|ca| ca.into_series()),
@@ -114,9 +117,13 @@ impl BinaryExpr {
         let left_s = ac_l.series().rechunk();
         let right_s = ac_r.series().rechunk();
         let res_s = apply_operator(&left_s, &right_s, self.op)?;
-        let ca = ListChunked::full(&name, &res_s, ac_l.groups.len());
         ac_l.with_update_groups(UpdateGroups::WithSeriesLen);
-        ac_l.with_series(ca.into_series(), true, Some(&self.expr))?;
+        let res_s = if res_s.len() == 1 {
+            res_s.new_from_index(0, ac_l.groups.len())
+        } else {
+            ListChunked::full(&name, &res_s, ac_l.groups.len()).into_series()
+        };
+        ac_l.with_series(res_s, true, Some(&self.expr))?;
         Ok(ac_l)
     }
 
@@ -136,22 +143,8 @@ impl BinaryExpr {
                 .with_name(&name)
         };
 
-        // Try if we can reuse the groups.
-        use AggState::*;
-        match (ac_l.agg_state(), ac_r.agg_state()) {
-            // No need to change update groups.
-            (AggregatedList(_), _) => {},
-            // We can take the groups of the rhs.
-            (_, AggregatedList(_)) if matches!(ac_r.update_groups, UpdateGroups::No) => {
-                ac_l.groups = ac_r.groups
-            },
-            // We must update the groups.
-            _ => {
-                ac_l.with_update_groups(UpdateGroups::WithSeriesLen);
-            },
-        }
-
-        ac_l.with_series(ca.into_series(), true, Some(&self.expr))?;
+        ac_l.with_update_groups(UpdateGroups::WithSeriesLen);
+        ac_l.with_agg_state(AggState::AggregatedList(ca.into_series()));
         Ok(ac_l)
     }
 }
@@ -197,7 +190,7 @@ impl PhysicalExpr for BinaryExpr {
         polars_ensure!(
             lhs.len() == rhs.len() || lhs.len() == 1 || rhs.len() == 1,
             expr = self.expr,
-            ComputeError: "cannot evaluate two series of different lengths ({} and {})",
+            ComputeError: "cannot evaluate two Series of different lengths ({} and {})",
             lhs.len(), rhs.len(),
         );
         apply_operator_owned(lhs, rhs, self.op)
@@ -261,13 +254,6 @@ impl PhysicalExpr for BinaryExpr {
     fn as_stats_evaluator(&self) -> Option<&dyn polars_io::predicates::StatsEvaluator> {
         Some(self)
     }
-
-    fn is_valid_aggregation(&self) -> bool {
-        // We don't want: col(a) == lit(1).
-        // We do want col(a).sum() == lit(1).
-        (!self.left.is_literal() && self.left.is_valid_aggregation())
-            || (!self.right.is_literal() && self.right.is_valid_aggregation())
-    }
 }
 
 #[cfg(feature = "parquet")]
@@ -400,15 +386,15 @@ mod stats {
             {
                 match (fld_l.data_type(), fld_r.data_type()) {
                     #[cfg(feature = "dtype-categorical")]
-                    (DataType::Utf8, DataType::Categorical(_)) => {},
+                    (DataType::String, DataType::Categorical(_, _) | DataType::Enum(_, _)) => {},
                     #[cfg(feature = "dtype-categorical")]
-                    (DataType::Categorical(_), DataType::Utf8) => {},
+                    (DataType::Categorical(_, _) | DataType::Enum(_, _), DataType::String) => {},
                     (l, r) if l != r => panic!("implementation error: {l:?}, {r:?}"),
                     _ => {},
                 }
             }
 
-            let dummy = DataFrame::new_no_checks(vec![]);
+            let dummy = DataFrame::empty();
             let state = ExecutionState::new();
 
             let out = match (self.left.is_literal(), self.right.is_literal()) {
@@ -456,13 +442,14 @@ mod stats {
                 return Ok(true);
             }
 
+            use Operator::*;
             match (
                 self.left.as_stats_evaluator(),
                 self.right.as_stats_evaluator(),
             ) {
                 (Some(l), Some(r)) => match self.op {
-                    Operator::And => Ok(l.should_read(stats)? && r.should_read(stats)?),
-                    Operator::Or => Ok(l.should_read(stats)? || r.should_read(stats)?),
+                    And | LogicalAnd => Ok(l.should_read(stats)? && r.should_read(stats)?),
+                    Or | LogicalOr => Ok(l.should_read(stats)? || r.should_read(stats)?),
                     _ => Ok(true),
                 },
                 _ => self.impl_should_read(stats),
