@@ -3,7 +3,7 @@ use std::ops::Deref;
 
 use polars_core::prelude::*;
 use polars_core::utils::NoNull;
-use polars_plan::dsl::consts::LITERAL_NAME;
+use polars_plan::constants::LITERAL_NAME;
 
 use crate::physical_plan::state::ExecutionState;
 use crate::prelude::*;
@@ -93,6 +93,12 @@ impl PhysicalExpr for LiteralExpr {
                 .into_time()
                 .into_series(),
             Series(series) => series.deref().clone(),
+            lv @ (Int(_) | Float(_) | StrCat(_)) => polars_core::prelude::Series::from_any_values(
+                LITERAL_NAME,
+                &[lv.to_any_value().unwrap()],
+                false,
+            )
+            .unwrap(),
         };
         Ok(s)
     }
