@@ -2,7 +2,6 @@ mod aggregations;
 mod arity;
 #[cfg(all(feature = "strings", feature = "cse"))]
 mod cse;
-mod err_msg;
 #[cfg(feature = "parquet")]
 mod io;
 mod logical;
@@ -10,12 +9,13 @@ mod optimization_checks;
 mod predicate_queries;
 mod projection_queries;
 mod queries;
+mod schema;
 #[cfg(feature = "streaming")]
 mod streaming;
 #[cfg(all(feature = "strings", feature = "cse"))]
 mod tpch;
 
-fn get_arenas() -> (Arena<AExpr>, Arena<ALogicalPlan>) {
+fn get_arenas() -> (Arena<AExpr>, Arena<IR>) {
     let expr_arena = Arena::with_capacity(16);
     let lp_arena = Arena::with_capacity(8);
     (expr_arena, lp_arena)
@@ -40,9 +40,6 @@ use polars_core::prelude::*;
 #[cfg(feature = "parquet")]
 pub(crate) use polars_core::SINGLE_LOCK;
 use polars_io::prelude::*;
-use polars_plan::logical_plan::{
-    OptimizationRule, SimplifyExprRule, StackOptimizer, TypeCoercionRule,
-};
 
 #[cfg(feature = "cov")]
 use crate::dsl::pearson_corr;
@@ -166,7 +163,7 @@ pub(crate) fn fruits_cars() -> DataFrame {
 
 pub(crate) fn get_df() -> DataFrame {
     let s = r#"
-"sepal.length","sepal.width","petal.length","petal.width","variety"
+"sepal_length","sepal_width","petal_length","petal_width","variety"
 5.1,3.5,1.4,.2,"Setosa"
 4.9,3,1.4,.2,"Setosa"
 4.7,3.2,1.3,.2,"Setosa"
