@@ -1,7 +1,7 @@
 use std::borrow::Cow;
 use std::fmt::Debug;
 
-use arrow::array::{Array, BinaryViewArray, View, INLINE_VIEW_SIZE};
+use arrow::array::{Array, BinaryViewArray, View};
 use arrow::bitmap::MutableBitmap;
 use arrow::buffer::Buffer;
 use arrow::legacy::trusted_len::TrustedLenPush;
@@ -113,7 +113,7 @@ impl TakeChunked for Series {
                 let ca = phys.str().unwrap();
                 let ca = ca.as_binary();
                 let out = take_unchecked_binview(&ca, by, sorted);
-                out.to_string().into_series()
+                out.to_string_unchecked().into_series()
             },
             List(_) => {
                 let ca = phys.list().unwrap();
@@ -169,7 +169,7 @@ impl TakeChunked for Series {
                 let ca = phys.str().unwrap();
                 let ca = ca.as_binary();
                 let out = take_unchecked_binview_opt(&ca, by);
-                out.to_string().into_series()
+                out.to_string_unchecked().into_series()
             },
             List(_) => {
                 let ca = phys.list().unwrap();
@@ -321,13 +321,25 @@ unsafe fn take_opt_unchecked_object(s: &Series, by: &[NullableChunkId]) -> Serie
 
 #[allow(clippy::unnecessary_cast)]
 #[inline(always)]
-fn rewrite_view(mut view: View, chunk_idx: IdxSize) -> View {
-    let chunk_idx = chunk_idx as u32;
-    let offset = [0, chunk_idx][(view.length > INLINE_VIEW_SIZE) as usize];
-    view.buffer_idx += offset;
+unsafe fn rewrite_view(mut view: View, chunk_idx: IdxSize, buffer_offsets: &[u32]) -> View {
+    if view.length > 12 {
+        let base_offset = *buffer_offsets.get_unchecked_release(chunk_idx as usize);
+        view.buffer_idx += base_offset;
+    }
     view
 }
 
+fn create_buffer_offsets(ca: &BinaryChunked) -> Vec<u32> {
+    let mut buffer_offsets = Vec::with_capacity(ca.chunks().len() + 1);
+    let mut cumsum = 0u32;
+    buffer_offsets.push(cumsum);
+    buffer_offsets.extend(ca.downcast_iter().map(|arr| {
+        cumsum += arr.data_buffers().len() as u32;
+        cumsum
+    }));
+    buffer_offsets
+}
+
 #[allow(clippy::unnecessary_cast)]
 unsafe fn take_unchecked_binview(
     ca: &BinaryChunked,
@@ -338,6 +350,8 @@ unsafe fn take_unchecked_binview(
         .downcast_iter()
         .map(|arr| arr.views().as_slice())
         .collect::<Vec<_>>();
+    let buffer_offsets = create_buffer_offsets(ca);
+
     let buffers: Arc<[Buffer<u8>]> = ca
         .downcast_iter()
         .flat_map(|arr| arr.data_buffers().as_ref())
@@ -353,8 +367,7 @@ unsafe fn take_unchecked_binview(
 
                 let target = *views.get_unchecked_release(chunk_idx as usize);
                 let view = *target.get_unchecked_release(array_idx);
-
-                rewrite_view(view, chunk_idx)
+                rewrite_view(view, chunk_idx, &buffer_offsets)
             })
             .collect::<Vec<_>>();
 
@@ -376,7 +389,7 @@ unsafe fn take_unchecked_binview(
             } else {
                 let target = *views.get_unchecked_release(chunk_idx as usize);
                 let view = *target.get_unchecked_release(array_idx);
-                let view = rewrite_view(view, chunk_idx);
+                let view = rewrite_view(view, chunk_idx, &buffer_offsets);
                 mut_views.push_unchecked(view);
                 validity.push_unchecked(true)
             }
@@ -410,6 +423,7 @@ unsafe fn take_unchecked_binview_opt(ca: &BinaryChunked, by: &[NullableChunkId])
         .flat_map(|arr| arr.data_buffers().as_ref())
         .cloned()
         .collect();
+    let buffer_offsets = create_buffer_offsets(ca);
 
     let targets = ca.downcast_iter().collect::<Vec<_>>();
 
@@ -427,7 +441,7 @@ unsafe fn take_unchecked_binview_opt(ca: &BinaryChunked, by: &[NullableChunkId])
 
                 let target = *views.get_unchecked_release(chunk_idx as usize);
                 let view = *target.get_unchecked_release(array_idx);
-                let view = rewrite_view(view, chunk_idx);
+                let view = rewrite_view(view, chunk_idx, &buffer_offsets);
 
                 mut_views.push_unchecked(view);
                 validity.push_unchecked(true)
@@ -450,7 +464,7 @@ unsafe fn take_unchecked_binview_opt(ca: &BinaryChunked, by: &[NullableChunkId])
                 } else {
                     let target = *views.get_unchecked_release(chunk_idx as usize);
                     let view = *target.get_unchecked_release(array_idx);
-                    let view = rewrite_view(view, chunk_idx);
+                    let view = rewrite_view(view, chunk_idx, &buffer_offsets);
                     mut_views.push_unchecked(view);
                     validity.push_unchecked(true);
                 }
@@ -482,18 +496,24 @@ mod test {
             // # Series without nulls;
             let mut s_1 = Series::new(
                 "a",
-                &["1 loooooooooooong string 1", "2 loooooooooooong string 2"],
+                &["1 loooooooooooong string", "2 loooooooooooong string"],
             );
             let s_2 = Series::new(
+                "a",
+                &["11 loooooooooooong string", "22 loooooooooooong string"],
+            );
+            let s_3 = Series::new(
                 "a",
                 &[
-                    "11 loooooooooooong string 11",
-                    "22 loooooooooooong string 22",
+                    "111 loooooooooooong string",
+                    "222 loooooooooooong string",
+                    "small", // this tests we don't mess with the inlined view
                 ],
             );
             s_1.append(&s_2).unwrap();
+            s_1.append(&s_3).unwrap();
 
-            assert_eq!(s_1.n_chunks(), 2);
+            assert_eq!(s_1.n_chunks(), 3);
 
             // ## Ids without nulls;
             let by = [
@@ -501,10 +521,13 @@ mod test {
                 ChunkId::store(0, 1),
                 ChunkId::store(1, 1),
                 ChunkId::store(1, 0),
+                ChunkId::store(2, 0),
+                ChunkId::store(2, 1),
+                ChunkId::store(2, 2),
             ];
 
             let out = s_1.take_chunked_unchecked(&by, IsSorted::Not);
-            let idx = IdxCa::new("", [0, 1, 3, 2]);
+            let idx = IdxCa::new("", [0, 1, 3, 2, 4, 5, 6]);
             let expected = s_1.rechunk().take(&idx).unwrap();
             assert!(out.equals(&expected));
 
