@@ -8,6 +8,7 @@ use polars_core::prelude::{DataType, SchemaRef, Series, IDX_DTYPE};
 use polars_core::schema::Schema;
 use polars_io::predicates::PhysicalIoExpr;
 use polars_plan::dsl::Expr;
+use polars_plan::logical_plan::expr_ir::ExprIR;
 use polars_plan::logical_plan::{ArenaExprIter, Context};
 use polars_plan::prelude::{AAggExpr, AExpr};
 use polars_utils::arena::{Arena, Node};
@@ -124,7 +125,7 @@ pub(crate) fn convert_to_hash_agg<F>(
     to_physical: &F,
 ) -> (DataType, Arc<dyn PhysicalPipedExpr>, AggregateFunction)
 where
-    F: Fn(Node, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
+    F: Fn(&ExprIR, &Arena<AExpr>, Option<&SchemaRef>) -> PolarsResult<Arc<dyn PhysicalPipedExpr>>,
 {
     match expr_arena.get(node) {
         AExpr::Alias(input, _) => convert_to_hash_agg(*input, expr_arena, schema, to_physical),
@@ -135,7 +136,12 @@ where
         ),
         AExpr::Agg(agg) => match agg {
             AAggExpr::Min { input, .. } => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
 
                 let agg_fn = match logical_dtype.to_physical() {
@@ -154,7 +160,12 @@ where
                 (logical_dtype, phys_expr, agg_fn)
             },
             AAggExpr::Max { input, .. } => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
 
                 let agg_fn = match logical_dtype.to_physical() {
@@ -173,7 +184,12 @@ where
                 (logical_dtype, phys_expr, agg_fn)
             },
             AAggExpr::Sum(input) => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
 
                 #[cfg(feature = "dtype-categorical")]
@@ -214,7 +230,12 @@ where
                 (logical_dtype, phys_expr, agg_fn)
             },
             AAggExpr::Mean(input) => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
 
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
                 #[cfg(feature = "dtype-categorical")]
@@ -229,7 +250,9 @@ where
                     );
                 }
                 let agg_fn = match logical_dtype.to_physical() {
-                    dt if dt.is_integer() => AggregateFunction::MeanF64(MeanAgg::<f64>::new()),
+                    dt if dt.is_integer() | dt.is_bool() => {
+                        AggregateFunction::MeanF64(MeanAgg::<f64>::new())
+                    },
                     DataType::Float32 => AggregateFunction::MeanF32(MeanAgg::<f32>::new()),
                     DataType::Float64 => AggregateFunction::MeanF64(MeanAgg::<f64>::new()),
                     dt => AggregateFunction::Null(NullAgg::new(dt)),
@@ -237,7 +260,12 @@ where
                 (logical_dtype, phys_expr, agg_fn)
             },
             AAggExpr::First(input) => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
                 (
                     logical_dtype.clone(),
@@ -246,7 +274,12 @@ where
                 )
             },
             AAggExpr::Last(input) => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
                 (
                     logical_dtype.clone(),
@@ -255,7 +288,12 @@ where
                 )
             },
             AAggExpr::Count(input, _) => {
-                let phys_expr = to_physical(*input, expr_arena, Some(schema)).unwrap();
+                let phys_expr = to_physical(
+                    &ExprIR::from_node(*input, expr_arena),
+                    expr_arena,
+                    Some(schema),
+                )
+                .unwrap();
                 let logical_dtype = phys_expr.field(schema).unwrap().dtype;
                 (
                     logical_dtype,
