@@ -4,21 +4,22 @@ use std::path::PathBuf;
 use std::sync::Arc;
 
 use arrow::datatypes::ArrowSchemaRef;
-use polars_core::config::{env_force_async, get_file_prefetch_size};
+use polars_core::config::{self, get_file_prefetch_size};
 use polars_core::error::*;
 use polars_core::prelude::Series;
 use polars_core::POOL;
 use polars_io::cloud::CloudOptions;
-use polars_io::parquet::{BatchedParquetReader, FileMetaData, ParquetReader};
+use polars_io::parquet::metadata::FileMetaDataRef;
+use polars_io::parquet::read::{BatchedParquetReader, ParquetOptions, ParquetReader};
 use polars_io::pl_async::get_runtime;
 use polars_io::predicates::PhysicalIoExpr;
 use polars_io::prelude::materialize_projection;
 #[cfg(feature = "async")]
 use polars_io::prelude::ParquetAsyncReader;
-use polars_io::utils::check_projected_arrow_schema;
-use polars_io::{is_cloud_url, SerReader};
+use polars_io::utils::{check_projected_arrow_schema, is_cloud_url};
+use polars_io::SerReader;
 use polars_plan::logical_plan::FileInfo;
-use polars_plan::prelude::{FileScanOptions, ParquetOptions};
+use polars_plan::prelude::FileScanOptions;
 use polars_utils::iter::EnumerateIdxTrait;
 use polars_utils::IdxSize;
 
@@ -36,7 +37,7 @@ pub struct ParquetSource {
     file_options: FileScanOptions,
     #[allow(dead_code)]
     cloud_options: Option<CloudOptions>,
-    metadata: Option<Arc<FileMetaData>>,
+    metadata: Option<FileMetaDataRef>,
     file_info: FileInfo,
     verbose: bool,
     run_async: bool,
@@ -190,7 +191,7 @@ impl ParquetSource {
         paths: Arc<[PathBuf]>,
         options: ParquetOptions,
         cloud_options: Option<CloudOptions>,
-        metadata: Option<Arc<FileMetaData>>,
+        metadata: Option<FileMetaDataRef>,
         file_options: FileScanOptions,
         file_info: FileInfo,
         verbose: bool,
@@ -204,7 +205,7 @@ impl ParquetSource {
         if verbose {
             eprintln!("POLARS PREFETCH_SIZE: {}", prefetch_size)
         }
-        let run_async = paths.first().map(is_cloud_url).unwrap_or(false) || env_force_async();
+        let run_async = paths.first().map(is_cloud_url).unwrap_or(false) || config::force_async();
 
         let mut source = ParquetSource {
             batched_readers: VecDeque::new(),
@@ -254,7 +255,9 @@ impl ParquetSource {
                     let init_iter = range.into_iter().map(|index| self.init_reader_async(index));
 
                     let batched_readers = polars_io::pl_async::get_runtime()
-                        .block_on(async { futures::future::try_join_all(init_iter).await })?;
+                        .block_on_potential_spawn(async {
+                            futures::future::try_join_all(init_iter).await
+                        })?;
 
                     for r in batched_readers {
                         self.finish_init_reader(r)?;
