@@ -7,6 +7,8 @@ mod array;
 mod binary;
 mod boolean;
 mod bounds;
+#[cfg(feature = "business")]
+mod business;
 #[cfg(feature = "dtype-categorical")]
 mod cat;
 #[cfg(feature = "round_series")]
@@ -23,6 +25,8 @@ mod datetime;
 mod dispatch;
 #[cfg(feature = "ewma")]
 mod ewm;
+#[cfg(feature = "ewma_by")]
+mod ewm_by;
 mod fill_null;
 #[cfg(feature = "fused")]
 mod fused;
@@ -81,6 +85,8 @@ use serde::{Deserialize, Serialize};
 
 pub(crate) use self::binary::BinaryFunction;
 pub use self::boolean::BooleanFunction;
+#[cfg(feature = "business")]
+pub(super) use self::business::BusinessFunction;
 #[cfg(feature = "dtype-categorical")]
 pub(crate) use self::cat::CategoricalFunction;
 #[cfg(feature = "temporal")]
@@ -93,7 +99,7 @@ pub(super) use self::rolling::RollingFunction;
 #[cfg(feature = "strings")]
 pub(crate) use self::strings::StringFunction;
 #[cfg(feature = "dtype-struct")]
-pub(super) use self::struct_::StructFunction;
+pub(crate) use self::struct_::StructFunction;
 #[cfg(feature = "trigonometry")]
 pub(super) use self::trigonometry::TrigonometricFunction;
 use super::*;
@@ -117,6 +123,8 @@ pub enum FunctionExpr {
 
     // Other expressions
     Boolean(BooleanFunction),
+    #[cfg(feature = "business")]
+    Business(BusinessFunction),
     #[cfg(feature = "abs")]
     Abs,
     Negate,
@@ -144,9 +152,7 @@ pub enum FunctionExpr {
     Atan2,
     #[cfg(feature = "sign")]
     Sign,
-    FillNull {
-        super_type: DataType,
-    },
+    FillNull,
     FillNullWithStrategy(FillNullStrategy),
     #[cfg(feature = "rolling_window")]
     RollingExpr(RollingFunction),
@@ -307,6 +313,11 @@ pub enum FunctionExpr {
     EwmMean {
         options: EWMOptions,
     },
+    #[cfg(feature = "ewma_by")]
+    EwmMeanBy {
+        half_life: Duration,
+        check_sorted: bool,
+    },
     #[cfg(feature = "ewma")]
     EwmStd {
         options: EWMOptions,
@@ -349,6 +360,8 @@ impl Hash for FunctionExpr {
 
             // Other expressions
             Boolean(f) => f.hash(state),
+            #[cfg(feature = "business")]
+            Business(f) => f.hash(state),
             Pow(f) => f.hash(state),
             #[cfg(feature = "search_sorted")]
             SearchSorted(f) => f.hash(state),
@@ -396,7 +409,7 @@ impl Hash for FunctionExpr {
             Sign => {},
             #[cfg(feature = "row_hash")]
             Hash(a, b, c, d) => (a, b, c, d).hash(state),
-            FillNull { super_type } => super_type.hash(state),
+            FillNull => {},
             #[cfg(feature = "rolling_window")]
             RollingExpr(f) => {
                 f.hash(state);
@@ -512,6 +525,11 @@ impl Hash for FunctionExpr {
             BackwardFill { limit } | ForwardFill { limit } => limit.hash(state),
             #[cfg(feature = "ewma")]
             EwmMean { options } => options.hash(state),
+            #[cfg(feature = "ewma_by")]
+            EwmMeanBy {
+                half_life,
+                check_sorted,
+            } => (half_life, check_sorted).hash(state),
             #[cfg(feature = "ewma")]
             EwmStd { options } => options.hash(state),
             #[cfg(feature = "ewma")]
@@ -557,6 +575,8 @@ impl Display for FunctionExpr {
 
             // Other expressions
             Boolean(func) => return write!(f, "{func}"),
+            #[cfg(feature = "business")]
+            Business(func) => return write!(f, "{func}"),
             #[cfg(feature = "abs")]
             Abs => "abs",
             Negate => "negate",
@@ -695,6 +715,8 @@ impl Display for FunctionExpr {
             MeanHorizontal => "mean_horizontal",
             #[cfg(feature = "ewma")]
             EwmMean { .. } => "ewm_mean",
+            #[cfg(feature = "ewma_by")]
+            EwmMeanBy { .. } => "ewm_mean_by",
             #[cfg(feature = "ewma")]
             EwmStd { .. } => "ewm_std",
             #[cfg(feature = "ewma")]
@@ -718,6 +740,14 @@ macro_rules! wrap {
     ($e:expr) => {
         SpecialEq::new(Arc::new($e))
     };
+
+    ($e:expr, $($args:expr),*) => {{
+        let f = move |s: &mut [Series]| {
+            $e(s, $($args),*)
+        };
+
+        SpecialEq::new(Arc::new(f))
+    }};
 }
 
 // Fn(&[Series], args)
@@ -807,6 +837,8 @@ impl From<FunctionExpr> for SpecialEq<Arc<dyn SeriesUdf>> {
 
             // Other expressions
             Boolean(func) => func.into(),
+            #[cfg(feature = "business")]
+            Business(func) => func.into(),
             #[cfg(feature = "abs")]
             Abs => map!(abs::abs),
             Negate => map!(dispatch::negate),
@@ -855,8 +887,8 @@ impl From<FunctionExpr> for SpecialEq<Arc<dyn SeriesUdf>> {
             Sign => {
                 map!(sign::sign)
             },
-            FillNull { super_type } => {
-                map_as_slice!(fill_null::fill_null, &super_type)
+            FillNull => {
+                map_as_slice!(fill_null::fill_null)
             },
             #[cfg(feature = "rolling_window")]
             RollingExpr(f) => {
@@ -1053,6 +1085,11 @@ impl From<FunctionExpr> for SpecialEq<Arc<dyn SeriesUdf>> {
             MeanHorizontal => wrap!(dispatch::mean_horizontal),
             #[cfg(feature = "ewma")]
             EwmMean { options } => map!(ewm::ewm_mean, options),
+            #[cfg(feature = "ewma_by")]
+            EwmMeanBy {
+                half_life,
+                check_sorted,
+            } => map_as_slice!(ewm_by::ewm_mean_by, half_life, check_sorted),
             #[cfg(feature = "ewma")]
             EwmStd { options } => map!(ewm::ewm_std, options),
             #[cfg(feature = "ewma")]
