@@ -2,6 +2,8 @@
 //!
 //! Functions on expressions that might be useful.
 mod arity;
+#[cfg(feature = "business")]
+mod business;
 #[cfg(feature = "dtype-struct")]
 mod coerce;
 mod concat;
@@ -18,6 +20,8 @@ mod syntactic_sugar;
 mod temporal;
 
 pub use arity::*;
+#[cfg(all(feature = "business", feature = "dtype-date"))]
+pub use business::*;
 #[cfg(feature = "dtype-struct")]
 pub use coerce::*;
 pub use concat::*;
