@@ -1,17 +1,20 @@
+#[cfg(feature = "cse")]
 mod hash;
 mod schema;
+mod utils;
 
 use std::hash::{Hash, Hasher};
 
+#[cfg(feature = "cse")]
+pub(super) use hash::traverse_and_hash_aexpr;
 use polars_core::prelude::*;
 use polars_core::utils::{get_time_units, try_get_supertype};
 use polars_utils::arena::{Arena, Node};
 use strum_macros::IntoStaticStr;
+pub use utils::*;
 
-#[cfg(feature = "cse")]
-use crate::logical_plan::visitor::AexprNode;
+use crate::constants::LEN;
 use crate::logical_plan::Context;
-use crate::prelude::consts::LEN;
 use crate::prelude::*;
 
 #[derive(Clone, Debug, IntoStaticStr)]
@@ -122,8 +125,8 @@ impl From<AAggExpr> for GroupByMethod {
 #[derive(Clone, Debug, Default)]
 pub enum AExpr {
     Explode(Node),
-    Alias(Node, Arc<str>),
-    Column(Arc<str>),
+    Alias(Node, ColumnName),
+    Column(ColumnName),
     Literal(LiteralValue),
     BinaryExpr {
         left: Node,
@@ -147,7 +150,7 @@ pub enum AExpr {
     SortBy {
         expr: Node,
         by: Vec<Node>,
-        descending: Vec<bool>,
+        sort_options: SortMultipleOptions,
     },
     Filter {
         input: Node,
@@ -160,14 +163,17 @@ pub enum AExpr {
         falsy: Node,
     },
     AnonymousFunction {
-        input: Vec<Node>,
+        input: Vec<ExprIR>,
         function: SpecialEq<Arc<dyn SeriesUdf>>,
         output_type: GetOutput,
         options: FunctionOptions,
     },
     Function {
-        /// function arguments
-        input: Vec<Node>,
+        /// Function arguments
+        /// Some functions rely on aliases,
+        /// for instance assignment of struct fields.
+        /// Therefore we need `[ExprIr]`.
+        input: Vec<ExprIR>,
         /// function to apply
         function: FunctionExpr,
         options: FunctionOptions,
@@ -189,21 +195,9 @@ pub enum AExpr {
 }
 
 impl AExpr {
-    #[cfg(feature = "cse")]
-    pub(crate) fn is_equal(l: Node, r: Node, arena: &Arena<AExpr>) -> bool {
-        let arena = arena as *const Arena<AExpr> as *mut Arena<AExpr>;
-        // SAFETY: we can pass a *mut pointer
-        // the equality operation will not access mutable
-        unsafe {
-            let ae_node_l = AexprNode::from_raw(l, arena);
-            let ae_node_r = AexprNode::from_raw(r, arena);
-            ae_node_l == ae_node_r
-        }
-    }
-
     #[cfg(feature = "cse")]
     pub(crate) fn col(name: &str) -> Self {
-        AExpr::Column(Arc::from(name))
+        AExpr::Column(ColumnName::from(name))
     }
     /// Any expression that is sensitive to the number of elements in a group
     /// - Aggregations
@@ -303,8 +297,7 @@ impl AExpr {
                 input
                     .iter()
                     .rev()
-                    .copied()
-                    .for_each(|node| container.push_node(node))
+                    .for_each(|e| container.push_node(e.node()))
             },
             Explode(e) => container.push_node(*e),
             Window {
@@ -383,8 +376,12 @@ impl AExpr {
                 return self;
             },
             AnonymousFunction { input, .. } | Function { input, .. } => {
-                input.clear();
-                input.extend(inputs.iter().rev().copied());
+                debug_assert_eq!(input.len(), inputs.len());
+
+                // Assign in reverse order as that was the order in which nodes were extracted.
+                for (e, node) in input.iter_mut().zip(inputs.iter().rev()) {
+                    e.set_node(*node);
+                }
                 return self;
             },
             Slice {
