@@ -1,6 +1,4 @@
 //! this contains code used for rewriting projections, expanding wildcards, regex selection etc.
-use polars_core::utils::get_supertype;
-
 use super::*;
 
 /// This replace the wildcard Expr with a Column Expr. It also removes the Exclude Expr from the
@@ -197,22 +195,6 @@ fn replace_dtype_with_column(expr: Expr, column_name: Arc<str>) -> Expr {
     })
 }
 
-fn set_null_st(e: Expr, schema: &Schema) -> Expr {
-    e.map_expr(|mut e| {
-        if let Expr::Function {
-            input,
-            function: FunctionExpr::FillNull { super_type },
-            ..
-        } = &mut e
-        {
-            if let Some(new_st) = early_supertype(input, schema) {
-                *super_type = new_st;
-            }
-        }
-        e
-    })
-}
-
 #[cfg(feature = "dtype-struct")]
 fn struct_index_to_field(expr: Expr, schema: &Schema) -> PolarsResult<Expr> {
     expr.try_map_expr(|e| match e {
@@ -394,34 +376,11 @@ fn expand_function_inputs(expr: Expr, schema: &Schema) -> Expr {
     })
 }
 
-/// this is determined in type coercion
-/// but checking a few types early can improve type stability (e.g. no need for unknown)
-fn early_supertype(inputs: &[Expr], schema: &Schema) -> Option<DataType> {
-    let mut arena = Arena::with_capacity(8);
-
-    let mut st = None;
-    for e in inputs {
-        let dtype = e
-            .to_field_amortized(schema, Context::Default, &mut arena)
-            .ok()?
-            .dtype;
-        arena.clear();
-        match st {
-            None => {
-                st = Some(dtype);
-            },
-            Some(st_val) => st = get_supertype(&st_val, &dtype),
-        }
-    }
-    st
-}
-
 #[derive(Copy, Clone)]
 struct ExpansionFlags {
     multiple_columns: bool,
     has_nth: bool,
     has_wildcard: bool,
-    replace_fill_null_type: bool,
     has_selector: bool,
     has_exclude: bool,
     #[cfg(feature = "dtype-struct")]
@@ -432,7 +391,6 @@ fn find_flags(expr: &Expr) -> ExpansionFlags {
     let mut multiple_columns = false;
     let mut has_nth = false;
     let mut has_wildcard = false;
-    let mut replace_fill_null_type = false;
     let mut has_selector = false;
     let mut has_exclude = false;
     #[cfg(feature = "dtype-struct")]
@@ -446,10 +404,6 @@ fn find_flags(expr: &Expr) -> ExpansionFlags {
             Expr::Nth(_) => has_nth = true,
             Expr::Wildcard => has_wildcard = true,
             Expr::Selector(_) => has_selector = true,
-            Expr::Function {
-                function: FunctionExpr::FillNull { .. },
-                ..
-            } => replace_fill_null_type = true,
             #[cfg(feature = "dtype-struct")]
             Expr::Function {
                 function: FunctionExpr::StructExpr(StructFunction::FieldByIndex(_)),
@@ -465,7 +419,6 @@ fn find_flags(expr: &Expr) -> ExpansionFlags {
         multiple_columns,
         has_nth,
         has_wildcard,
-        replace_fill_null_type,
         has_selector,
         has_exclude,
         #[cfg(feature = "dtype-struct")]
@@ -483,6 +436,7 @@ pub(crate) fn rewrite_projections(
     let mut result = Vec::with_capacity(exprs.len() + schema.len());
 
     for mut expr in exprs {
+        #[cfg(feature = "dtype-struct")]
         let result_offset = result.len();
 
         // Functions can have col(["a", "b"]) or col(String) as inputs.
@@ -497,18 +451,6 @@ pub(crate) fn rewrite_projections(
 
         replace_and_add_to_results(expr, flags, &mut result, schema, keys)?;
 
-        // This is done after all expansion (wildcard, column, dtypes)
-        // have been done. This will ensure the conversion to aexpr does
-        // not panic because of an unexpected wildcard etc.
-
-        // The expanded expressions are written to result, so we pick
-        // them up there.
-        if flags.replace_fill_null_type {
-            for e in &mut result[result_offset..] {
-                *e = set_null_st(std::mem::take(e), schema);
-            }
-        }
-
         #[cfg(feature = "dtype-struct")]
         if flags.has_struct_field_by_index {
             for e in &mut result[result_offset..] {
