@@ -1,5 +1,6 @@
-use std::borrow::Cow;
+use std::ops::Deref;
 use std::path::Path;
+use std::sync::Mutex;
 
 use arrow::datatypes::ArrowSchemaRef;
 use polars_core::prelude::*;
@@ -7,39 +8,28 @@ use polars_utils::format_smartstring;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
+use super::hive::HivePartitions;
 use crate::prelude::*;
 
-impl LogicalPlan {
-    pub fn schema(&self) -> PolarsResult<Cow<'_, SchemaRef>> {
-        use LogicalPlan::*;
-        match self {
-            Scan { file_info, .. } => Ok(Cow::Borrowed(&file_info.schema)),
-            #[cfg(feature = "python")]
-            PythonScan { options } => Ok(Cow::Borrowed(&options.schema)),
-            Union { inputs, .. } => inputs[0].schema(),
-            HConcat { schema, .. } => Ok(Cow::Borrowed(schema)),
-            Cache { input, .. } => input.schema(),
-            Sort { input, .. } => input.schema(),
-            DataFrameScan { schema, .. } => Ok(Cow::Borrowed(schema)),
-            Selection { input, .. } => input.schema(),
-            Projection { schema, .. } => Ok(Cow::Borrowed(schema)),
-            Aggregate { schema, .. } => Ok(Cow::Borrowed(schema)),
-            Join { schema, .. } => Ok(Cow::Borrowed(schema)),
-            HStack { schema, .. } => Ok(Cow::Borrowed(schema)),
-            Distinct { input, .. } | Sink { input, .. } => input.schema(),
-            Slice { input, .. } => input.schema(),
-            MapFunction {
-                input, function, ..
-            } => {
-                let input_schema = input.schema()?;
-                match input_schema {
-                    Cow::Owned(schema) => Ok(Cow::Owned(function.schema(&schema)?.into_owned())),
-                    Cow::Borrowed(schema) => function.schema(schema),
-                }
-            },
-            Error { err, .. } => Err(err.take()),
-            ExtContext { schema, .. } => Ok(Cow::Borrowed(schema)),
-        }
+impl DslPlan {
+    pub fn compute_schema(&self) -> PolarsResult<SchemaRef> {
+        let opt_state = OptState {
+            eager: true,
+            type_coercion: true,
+            simplify_expr: false,
+            ..Default::default()
+        };
+
+        let mut lp_arena = Default::default();
+        let node = optimize(
+            self.clone(),
+            opt_state,
+            &mut lp_arena,
+            &mut Default::default(),
+            &mut Default::default(),
+            Default::default(),
+        )?;
+        Ok(lp_arena.get(node).schema(&lp_arena).into_owned())
     }
 }
 
@@ -53,10 +43,11 @@ pub struct FileInfo {
     /// - known size
     /// - estimated size
     pub row_estimation: (Option<usize>, usize),
-    pub hive_parts: Option<Arc<hive::HivePartitions>>,
+    pub hive_parts: Option<Arc<HivePartitions>>,
 }
 
 impl FileInfo {
+    /// Constructs a new [`FileInfo`].
     pub fn new(
         schema: SchemaRef,
         reader_schema: Option<ArrowSchemaRef>,
@@ -70,35 +61,57 @@ impl FileInfo {
         }
     }
 
-    /// Updates the statistics and merges the hive partitions schema with the file one.
-    pub fn init_hive_partitions(&mut self, url: &Path) -> PolarsResult<()> {
-        self.hive_parts = hive::HivePartitions::parse_url(url).map(|hive_parts| {
-            let hive_schema = hive_parts.get_statistics().schema().clone();
-            let expected_len = self.schema.len() + hive_schema.len();
-
-            let schema = Arc::make_mut(&mut self.schema);
-            schema.merge((**hive_parts.get_statistics().schema()).clone());
-
-            polars_ensure!(schema.len() == expected_len, ComputeError: "invalid hive partitions\n\n\
-            Extending the schema with the hive partitioned columns creates duplicate fields.");
-
-            Ok(Arc::new(hive_parts))
-        }).transpose()?;
+    /// Set the [`HivePartitions`] information for this [`FileInfo`] from a path and an
+    /// optional schema.
+    pub fn init_hive_partitions(
+        &mut self,
+        path: &Path,
+        schema: Option<SchemaRef>,
+    ) -> PolarsResult<()> {
+        let hp = HivePartitions::try_from_path(path, schema)?;
+        if let Some(hp) = hp {
+            let hive_schema = hp.schema().clone();
+            self.update_schema_with_hive_schema(hive_schema)?;
+            self.hive_parts = Some(Arc::new(hp));
+        }
         Ok(())
     }
 
-    /// Updates the statistics, but not the schema.
-    pub fn update_hive_partitions(&mut self, url: &Path) -> PolarsResult<()> {
+    /// Merge the [`Schema`] of a [`HivePartitions`] with the schema of this [`FileInfo`].
+    ///
+    /// Returns an `Err` if any of the columns in either schema overlap.
+    fn update_schema_with_hive_schema(&mut self, hive_schema: SchemaRef) -> PolarsResult<()> {
+        let expected_len = self.schema.len() + hive_schema.len();
+
+        let file_schema = Arc::make_mut(&mut self.schema);
+        file_schema.merge(Arc::unwrap_or_clone(hive_schema));
+
+        polars_ensure!(
+            file_schema.len() == expected_len,
+            Duplicate: "invalid Hive partition schema\n\n\
+            Extending the schema with the Hive partition schema would create duplicate fields."
+        );
+        Ok(())
+    }
+
+    /// Update the [`HivePartitions`] statistics for this [`FileInfo`].
+    ///
+    /// If the Hive partitions were not yet initialized, this function has no effect.
+    pub fn update_hive_partitions(&mut self, path: &Path) -> PolarsResult<()> {
         if let Some(current) = &mut self.hive_parts {
-            let new = hive::HivePartitions::parse_url(url).ok_or_else(|| polars_err!(ComputeError: "expected hive partitioned path, got {}\n\n\
-            This error occurs if 'hive_partitioning=true' some paths are hive partitioned and some paths are not.", url.display()))?;
+            let schema = current.schema().clone();
+            let hp = HivePartitions::try_from_path(path, Some(schema))?;
+            let Some(new) = hp else {
+                polars_bail!(
+                    ComputeError: "expected Hive partitioned path, got {}\n\n\
+                    This error occurs if some paths are Hive partitioned and some paths are not.",
+                    path.display()
+                )
+            };
+
             match Arc::get_mut(current) {
-                Some(current) => {
-                    *current = new;
-                },
-                _ => {
-                    *current = Arc::new(new);
-                },
+                Some(hp) => *hp = new,
+                None => *current = Arc::new(new),
             }
         }
         Ok(())
@@ -124,12 +137,12 @@ fn estimate_sizes(
 #[cfg(feature = "streaming")]
 pub fn set_estimated_row_counts(
     root: Node,
-    lp_arena: &mut Arena<ALogicalPlan>,
+    lp_arena: &mut Arena<IR>,
     expr_arena: &Arena<AExpr>,
     mut _filter_count: usize,
     scratch: &mut Vec<Node>,
 ) -> (Option<usize>, usize, usize) {
-    use ALogicalPlan::*;
+    use IR::*;
 
     fn apply_slice(out: &mut (Option<usize>, usize, usize), slice: Option<(i64, usize)>) {
         if let Some((_, len)) = slice {
@@ -139,9 +152,9 @@ pub fn set_estimated_row_counts(
     }
 
     match lp_arena.get(root) {
-        Selection { predicate, input } => {
+        Filter { predicate, input } => {
             _filter_count += expr_arena
-                .iter(*predicate)
+                .iter(predicate.node())
                 .filter(|(_, ae)| matches!(ae, AExpr::BinaryExpr { .. }))
                 .count()
                 + 1;
@@ -361,3 +374,35 @@ pub(crate) fn det_join_schema(
         },
     }
 }
+
+// We don't use an `Arc<Mutex>` because caches should live in different query plans.
+// For that reason we have a specialized deep clone.
+#[derive(Default)]
+pub struct CachedSchema(Mutex<Option<SchemaRef>>);
+
+impl AsRef<Mutex<Option<SchemaRef>>> for CachedSchema {
+    fn as_ref(&self) -> &Mutex<Option<SchemaRef>> {
+        &self.0
+    }
+}
+
+impl Deref for CachedSchema {
+    type Target = Mutex<Option<SchemaRef>>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl Clone for CachedSchema {
+    fn clone(&self) -> Self {
+        let inner = self.0.lock().unwrap();
+        Self(Mutex::new(inner.clone()))
+    }
+}
+
+impl CachedSchema {
+    pub fn get(&self) -> Option<SchemaRef> {
+        self.0.lock().unwrap().clone()
+    }
+}
