@@ -1,3 +1,4 @@
+use polars_core::chunked_array::ops::SortMultipleOptions;
 use polars_core::df;
 use polars_lazy::prelude::*;
 use polars_sql::*;
@@ -29,9 +30,9 @@ fn test_distinct_on() {
     let expected = df
         .sort_by_exprs(
             vec![col("Name"), col("Record Date")],
-            vec![false, true],
-            true,
-            false,
+            SortMultipleOptions::default()
+                .with_order_descendings([false, true])
+                .with_maintain_order(true),
         )
         .group_by_stable(vec![col("Name")])
         .agg(vec![col("*").first()]);
