@@ -70,7 +70,7 @@ where
         })
     } else {
         if arr.null_count() > 0 {
-            panic!("'rolling by' not yet supported for series with null values, consider using 'group_by_rolling'")
+            polars_bail!(InvalidOperation: "'Expr.rolling_*(..., by=...)' not yet supported for series with null values, consider using 'DataFrame.rolling' or 'Expr.rolling'")
         }
         let values = arr.values().as_slice();
         let duration = options.window_size;
@@ -79,7 +79,7 @@ where
         let by = options.by.unwrap();
         let closed_window = options.closed_window.expect("closed window  must be set");
         let func = rolling_agg_fn_dynamic.expect(
-            "'rolling by' not yet supported for this expression, consider using 'group_by_rolling'",
+            "'Expr.rolling_*(..., by=...)' not yet supported for this expression, consider using 'DataFrame.rolling' or 'Expr.rolling'",
         );
 
         func(
