@@ -23,11 +23,3 @@ pub(crate) fn fruits_cars() -> DataFrame {
     )
     .unwrap()
 }
-
-fn load_df() -> DataFrame {
-    df!("a" => &[1, 2, 3, 4, 5],
-                 "b" => &["a", "a", "b", "c", "c"],
-                 "c" => &[1, 2, 3, 4, 5]
-    )
-    .unwrap()
-}
