@@ -1,11 +1,14 @@
+use std::borrow::Cow;
+
 #[cfg(feature = "object")]
 use polars::chunked_array::object::PolarsObjectSafe;
 use polars::datatypes::{DataType, Field, OwnedObject, PlHashMap, TimeUnit};
 use polars::prelude::{AnyValue, Series};
-use polars_core::frame::row::any_values_to_dtype;
+use polars_core::utils::any_values_to_supertype_and_n_dtypes;
+use pyo3::exceptions::{PyOverflowError, PyTypeError};
 use pyo3::intern;
 use pyo3::prelude::*;
-use pyo3::types::{PyBool, PyDict, PyFloat, PyList, PySequence, PyString, PyTuple, PyType};
+use pyo3::types::{PyBool, PyBytes, PyDict, PyFloat, PyInt, PyList, PySequence, PyString, PyTuple};
 
 use super::{decimal_to_digits, struct_dict, ObjectValue, Wrap};
 use crate::error::PyPolarsErr;
@@ -24,14 +27,14 @@ impl ToPyObject for Wrap<AnyValue<'_>> {
     }
 }
 
-impl<'s> FromPyObject<'s> for Wrap<AnyValue<'s>> {
-    fn extract(ob: &'s PyAny) -> PyResult<Self> {
+impl<'py> FromPyObject<'py> for Wrap<AnyValue<'py>> {
+    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
         py_object_to_any_value(ob, true).map(Wrap)
     }
 }
 
 pub(crate) fn any_value_into_py_object(av: AnyValue, py: Python) -> PyObject {
-    let utils = UTILS.as_ref(py);
+    let utils = UTILS.bind(py);
     match av {
         AnyValue::UInt8(v) => v.into_py(py),
         AnyValue::UInt16(v) => v.into_py(py),
@@ -102,7 +105,7 @@ pub(crate) fn any_value_into_py_object(av: AnyValue, py: Python) -> PyObject {
                     N * std::mem::size_of::<u128>(),
                 )
             };
-            let digits = PyTuple::new(py, buf.iter().take(n_digits));
+            let digits = PyTuple::new_bound(py, buf.iter().take(n_digits));
             convert
                 .call1((v.is_negative() as u8, digits, n_digits, -(scale as i32)))
                 .unwrap()
@@ -112,117 +115,69 @@ pub(crate) fn any_value_into_py_object(av: AnyValue, py: Python) -> PyObject {
 }
 
 type TypeObjectPtr = usize;
-type InitFn = fn(&PyAny, bool) -> PyResult<AnyValue>;
+type InitFn = for<'py> fn(&Bound<'py, PyAny>, bool) -> PyResult<AnyValue<'py>>;
 pub(crate) static LUT: crate::gil_once_cell::GILOnceCell<PlHashMap<TypeObjectPtr, InitFn>> =
     crate::gil_once_cell::GILOnceCell::new();
 
-pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyValue> {
-    // conversion functions
-    fn get_bool(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+pub(crate) fn py_object_to_any_value<'py>(
+    ob: &Bound<'py, PyAny>,
+    strict: bool,
+) -> PyResult<AnyValue<'py>> {
+    // Conversion functions.
+    fn get_null(_ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
+        Ok(AnyValue::Null)
+    }
+
+    fn get_bool(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         let b = ob.extract::<bool>().unwrap();
         Ok(AnyValue::Boolean(b))
     }
 
-    fn get_int(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
-        // can overflow
-        match ob.extract::<i64>() {
-            Ok(v) => Ok(AnyValue::Int64(v)),
-            Err(_) => Ok(AnyValue::UInt64(ob.extract::<u64>()?)),
+    fn get_int(ob: &Bound<'_, PyAny>, strict: bool) -> PyResult<AnyValue<'static>> {
+        if let Ok(v) = ob.extract::<i64>() {
+            Ok(AnyValue::Int64(v))
+        } else if let Ok(v) = ob.extract::<u64>() {
+            Ok(AnyValue::UInt64(v))
+        } else if !strict {
+            let f = ob.extract::<f64>()?;
+            Ok(AnyValue::Float64(f))
+        } else {
+            Err(PyOverflowError::new_err(format!(
+                "int value too large for Polars integer types: {ob}"
+            )))
         }
     }
 
-    fn get_float(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_float(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         Ok(AnyValue::Float64(ob.extract::<f64>().unwrap()))
     }
 
-    fn get_str(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
-        let value = ob.extract::<&str>().unwrap();
-        Ok(AnyValue::String(value))
+    fn get_str(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
+        // Ideally we'd be returning an AnyValue::String(&str) instead, as was
+        // the case in previous versions of this function. However, if compiling
+        // with abi3 for versions older than Python 3.10, the APIs that purport
+        // to return &str actually just encode to UTF-8 as a newly allocated
+        // PyBytes object, and then return reference to that. So what we're
+        // doing here isn't any different fundamantelly, and the APIs to for
+        // converting to &str are deprecated in PyO3 0.21.
+        //
+        // Once Python 3.10 is the minimum supported version, converting to &str
+        // will be cheaper, and we should do that. Python 3.9 security updates
+        // end-of-life is Oct 31, 2025.
+        Ok(AnyValue::StringOwned(
+            ob.extract::<String>().unwrap().into(),
+        ))
     }
 
-    fn get_struct(ob: &PyAny, strict: bool) -> PyResult<AnyValue<'_>> {
-        let dict = ob.downcast::<PyDict>().unwrap();
-        let len = dict.len();
-        let mut keys = Vec::with_capacity(len);
-        let mut vals = Vec::with_capacity(len);
-        for (k, v) in dict.into_iter() {
-            let key = k.extract::<&str>()?;
-            let val = py_object_to_any_value(v, strict)?;
-            let dtype = val.dtype();
-            keys.push(Field::new(key, dtype));
-            vals.push(val)
-        }
-        Ok(AnyValue::StructOwned(Box::new((vals, keys))))
-    }
-
-    fn get_list(ob: &PyAny, strict: bool) -> PyResult<AnyValue> {
-        fn get_list_with_constructor(ob: &PyAny) -> PyResult<AnyValue> {
-            // Use the dedicated constructor
-            // this constructor is able to go via dedicated type constructors
-            // so it can be much faster
-            Python::with_gil(|py| {
-                let s = SERIES.call1(py, (ob,))?;
-                get_series_el(s.as_ref(py), true)
-            })
-        }
-
-        if ob.is_empty()? {
-            Ok(AnyValue::List(Series::new_empty("", &DataType::Null)))
-        } else if ob.is_instance_of::<PyList>() | ob.is_instance_of::<PyTuple>() {
-            const INFER_SCHEMA_LENGTH: usize = 25;
-
-            let list = ob.downcast::<PySequence>().unwrap();
-
-            let mut avs = Vec::with_capacity(INFER_SCHEMA_LENGTH);
-            let mut iter = list.iter()?;
-
-            for item in (&mut iter).take(INFER_SCHEMA_LENGTH) {
-                let av = py_object_to_any_value(item?, strict)?;
-                avs.push(av)
-            }
-
-            let (dtype, n_types) = any_values_to_dtype(&avs).map_err(PyPolarsErr::from)?;
-
-            // we only take this path if there is no question of the data-type
-            if dtype.is_primitive() && n_types == 1 {
-                get_list_with_constructor(ob)
-            } else {
-                // push the rest
-                avs.reserve(list.len()?);
-                for item in iter {
-                    let av = py_object_to_any_value(item?, strict)?;
-                    avs.push(av)
-                }
-
-                let s = Series::from_any_values_and_dtype("", &avs, &dtype, strict)
-                    .map_err(PyPolarsErr::from)?;
-                Ok(AnyValue::List(s))
-            }
-        } else {
-            // range will take this branch
-            get_list_with_constructor(ob)
-        }
-    }
-
-    fn get_series_el(ob: &PyAny, _strict: bool) -> PyResult<AnyValue<'static>> {
-        let py_pyseries = ob.getattr(intern!(ob.py(), "_s")).unwrap();
-        let series = py_pyseries.extract::<PySeries>().unwrap().series;
-        Ok(AnyValue::List(series))
-    }
-
-    fn get_bin(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
-        let value = ob.extract::<&[u8]>().unwrap();
+    fn get_bytes<'py>(ob: &Bound<'py, PyAny>, _strict: bool) -> PyResult<AnyValue<'py>> {
+        let value = ob.extract::<&'py [u8]>().unwrap();
         Ok(AnyValue::Binary(value))
     }
 
-    fn get_null(_ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
-        Ok(AnyValue::Null)
-    }
-
-    fn get_date(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_date(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         Python::with_gil(|py| {
             let date = UTILS
-                .as_ref(py)
+                .bind(py)
                 .getattr(intern!(py, "date_to_int"))
                 .unwrap()
                 .call1((ob,))
@@ -232,10 +187,10 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
         })
     }
 
-    fn get_datetime(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_datetime(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         Python::with_gil(|py| {
             let date = UTILS
-                .as_ref(py)
+                .bind(py)
                 .getattr(intern!(py, "datetime_to_int"))
                 .unwrap()
                 .call1((ob, intern!(py, "us")))
@@ -245,10 +200,10 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
         })
     }
 
-    fn get_timedelta(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_timedelta(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         Python::with_gil(|py| {
             let td = UTILS
-                .as_ref(py)
+                .bind(py)
                 .getattr(intern!(py, "timedelta_to_int"))
                 .unwrap()
                 .call1((ob, intern!(py, "us")))
@@ -258,10 +213,10 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
         })
     }
 
-    fn get_time(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_time(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         Python::with_gil(|py| {
             let time = UTILS
-                .as_ref(py)
+                .bind(py)
                 .getattr(intern!(py, "time_to_int"))
                 .unwrap()
                 .call1((ob,))
@@ -271,7 +226,7 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
         })
     }
 
-    fn get_decimal(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
+    fn get_decimal(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
         fn abs_decimal_from_digits(
             digits: impl IntoIterator<Item = u8>,
             exp: i32,
@@ -285,9 +240,9 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
                     v = v.checked_mul(10).and_then(|v| v.checked_add(d))?;
                 }
             }
-            // we only support non-negative scale (=> non-positive exponent)
+            // We only support non-negative scale (=> non-positive exponent).
             let scale = if exp > 0 {
-                // the decimal may be in a non-canonical representation, try to fix it first
+                // The decimal may be in a non-canonical representation, try to fix it first.
                 v = 10_i128
                     .checked_pow(exp as u32)
                     .and_then(|factor| v.checked_mul(factor))?;
@@ -295,116 +250,189 @@ pub(crate) fn py_object_to_any_value(ob: &PyAny, strict: bool) -> PyResult<AnyVa
             } else {
                 (-exp) as usize
             };
-            // TODO: do we care for checking if it fits in MAX_ABS_DEC? (if we set precision to None anyway?)
+            // TODO: Do we care for checking if it fits in MAX_ABS_DEC? (if we set precision to None anyway?)
             (v <= MAX_ABS_DEC).then_some((v, scale))
         }
 
+        // Note: Using Vec<u8> is not the most efficient thing here (input is a tuple)
         let (sign, digits, exp): (i8, Vec<u8>, i32) = ob
             .call_method0(intern!(ob.py(), "as_tuple"))
             .unwrap()
             .extract()
             .unwrap();
-        // note: using Vec<u8> is not the most efficient thing here (input is a tuple)
         let (mut v, scale) = abs_decimal_from_digits(digits, exp).ok_or_else(|| {
             PyErr::from(PyPolarsErr::Other(
                 "Decimal is too large to fit in Decimal128".into(),
             ))
         })?;
         if sign > 0 {
-            v = -v; // won't overflow since -i128::MAX > i128::MIN
+            v = -v; // Won't overflow since -i128::MAX > i128::MIN
         }
         Ok(AnyValue::Decimal(v, scale))
     }
 
-    fn get_object(ob: &PyAny, _strict: bool) -> PyResult<AnyValue> {
-        #[cfg(feature = "object")]
-        {
-            // this is slow, but hey don't use objects
-            let v = &ObjectValue { inner: ob.into() };
-            Ok(AnyValue::ObjectOwned(OwnedObject(v.to_boxed())))
+    fn get_list(ob: &Bound<'_, PyAny>, strict: bool) -> PyResult<AnyValue<'static>> {
+        fn get_list_with_constructor(ob: &Bound<'_, PyAny>) -> PyResult<AnyValue<'static>> {
+            // Use the dedicated constructor.
+            // This constructor is able to go via dedicated type constructors
+            // so it can be much faster.
+            let py = ob.py();
+            let s = SERIES.call1(py, (ob,))?;
+            get_list_from_series(s.bind(py), true)
         }
-        #[cfg(not(feature = "object"))]
-        {
-            panic!("activate object")
+
+        if ob.is_empty()? {
+            Ok(AnyValue::List(Series::new_empty("", &DataType::Null)))
+        } else if ob.is_instance_of::<PyList>() | ob.is_instance_of::<PyTuple>() {
+            const INFER_SCHEMA_LENGTH: usize = 25;
+
+            let list = ob.downcast::<PySequence>().unwrap();
+
+            let mut avs = Vec::with_capacity(INFER_SCHEMA_LENGTH);
+            let mut iter = list.iter()?;
+            let mut items = Vec::with_capacity(INFER_SCHEMA_LENGTH);
+            for item in (&mut iter).take(INFER_SCHEMA_LENGTH) {
+                items.push(item?);
+                let av = py_object_to_any_value(items.last().unwrap(), strict)?;
+                avs.push(av)
+            }
+            let (dtype, n_dtypes) = any_values_to_supertype_and_n_dtypes(&avs)
+                .map_err(|e| PyTypeError::new_err(e.to_string()))?;
+
+            // This path is only taken if there is no question about the data type.
+            if dtype.is_primitive() && n_dtypes == 1 {
+                get_list_with_constructor(ob)
+            } else {
+                // Push the rest.
+                let length = list.len()?;
+                avs.reserve(length);
+                let mut rest = Vec::with_capacity(length);
+                for item in iter {
+                    rest.push(item?);
+                    let av = py_object_to_any_value(rest.last().unwrap(), strict)?;
+                    avs.push(av)
+                }
+
+                let s = Series::from_any_values_and_dtype("", &avs, &dtype, strict)
+                    .map_err(|e| {
+                        PyTypeError::new_err(format!(
+                            "{e}\n\nHint: Try setting `strict=False` to allow passing data with mixed types."
+                        ))
+                    })?;
+                Ok(AnyValue::List(s))
+            }
+        } else {
+            // range will take this branch
+            get_list_with_constructor(ob)
         }
     }
 
-    // TYPE key
-    let type_object_ptr = PyType::as_type_ptr(ob.get_type()) as usize;
+    fn get_list_from_series(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
+        let s = super::get_series(ob)?;
+        Ok(AnyValue::List(s))
+    }
+
+    fn get_struct<'py>(ob: &Bound<'py, PyAny>, strict: bool) -> PyResult<AnyValue<'py>> {
+        let dict = ob.downcast::<PyDict>().unwrap();
+        let len = dict.len();
+        let mut keys = Vec::with_capacity(len);
+        let mut vals = Vec::with_capacity(len);
+        for (k, v) in dict.into_iter() {
+            let key = k.extract::<Cow<str>>()?;
+            let val = py_object_to_any_value(&v, strict)?;
+            let dtype = val.dtype();
+            keys.push(Field::new(&key, dtype));
+            vals.push(val)
+        }
+        Ok(AnyValue::StructOwned(Box::new((vals, keys))))
+    }
+
+    fn get_object(ob: &Bound<'_, PyAny>, _strict: bool) -> PyResult<AnyValue<'static>> {
+        #[cfg(feature = "object")]
+        {
+            // This is slow, but hey don't use objects.
+            let v = &ObjectValue {
+                inner: ob.clone().unbind(),
+            };
+            Ok(AnyValue::ObjectOwned(OwnedObject(v.to_boxed())))
+        }
+        #[cfg(not(feature = "object"))]
+        panic!("activate object")
+    }
+
+    /// Determine which conversion function to use for the given object.
+    ///
+    /// Note: This function is only ran if the object's type is not already in the
+    /// lookup table.
+    fn get_conversion_function(ob: &Bound<'_, PyAny>, py: Python<'_>) -> InitFn {
+        if ob.is_none() {
+            get_null
+        }
+        // bool must be checked before int because Python bool is an instance of int.
+        else if ob.is_instance_of::<PyBool>() {
+            get_bool
+        } else if ob.is_instance_of::<PyInt>() {
+            get_int
+        } else if ob.is_instance_of::<PyFloat>() {
+            get_float
+        } else if ob.is_instance_of::<PyString>() {
+            get_str
+        } else if ob.is_instance_of::<PyBytes>() {
+            get_bytes
+        } else if ob.is_instance_of::<PyList>() || ob.is_instance_of::<PyTuple>() {
+            get_list
+        } else if ob.is_instance_of::<PyDict>() {
+            get_struct
+        } else if ob.hasattr(intern!(py, "_s")).unwrap() {
+            get_list_from_series
+        } else {
+            let type_name = ob.get_type().qualname().unwrap();
+            match &*type_name {
+                // Can't use pyo3::types::PyDateTime with abi3-py37 feature,
+                // so need this workaround instead of `isinstance(ob, datetime)`.
+                "date" => get_date as InitFn,
+                "time" => get_time as InitFn,
+                "datetime" => get_datetime as InitFn,
+                "timedelta" => get_timedelta as InitFn,
+                "Decimal" => get_decimal as InitFn,
+                "range" => get_list as InitFn,
+                _ => {
+                    // Support NumPy scalars.
+                    if ob.extract::<i64>().is_ok() || ob.extract::<u64>().is_ok() {
+                        return get_int as InitFn;
+                    } else if ob.extract::<f64>().is_ok() {
+                        return get_float as InitFn;
+                    }
+
+                    // Support custom subclasses of datetime/date.
+                    let ancestors = ob.get_type().getattr(intern!(py, "__mro__")).unwrap();
+                    let ancestors_str_iter = ancestors
+                        .iter()
+                        .unwrap()
+                        .map(|b| b.unwrap().str().unwrap().to_string());
+                    for c in ancestors_str_iter {
+                        match &*c {
+                            // datetime must be checked before date because
+                            // Python datetime is an instance of date.
+                            "<class 'datetime.datetime'>" => return get_datetime as InitFn,
+                            "<class 'datetime.date'>" => return get_date as InitFn,
+                            _ => (),
+                        }
+                    }
+
+                    get_object as InitFn
+                },
+            }
+        }
+    }
+
+    let type_object_ptr = ob.get_type().as_type_ptr() as usize;
 
     Python::with_gil(|py| {
         LUT.with_gil(py, |lut| {
-            // get the conversion function
-            let convert_fn = lut.entry(type_object_ptr).or_insert_with(
-                // This only runs if type is not in LUT
-                || {
-                    if ob.is_instance_of::<PyBool>() {
-                        get_bool
-                    // TODO: this heap allocs on failure
-                    } else if ob.extract::<i64>().is_ok() || ob.extract::<u64>().is_ok() {
-                        get_int
-                    } else if ob.is_instance_of::<PyFloat>() {
-                        get_float
-                    } else if ob.is_instance_of::<PyString>() {
-                        get_str
-                    } else if ob.is_instance_of::<PyDict>() {
-                        get_struct
-                    } else if ob.is_instance_of::<PyList>() || ob.is_instance_of::<PyTuple>() {
-                        get_list
-                    } else if ob.hasattr(intern!(py, "_s")).unwrap() {
-                        get_series_el
-                    }
-                    // TODO: this heap allocs on failure
-                    else if ob.extract::<&[u8]>().is_ok() {
-                        get_bin
-                    } else if ob.is_none() {
-                        get_null
-                    } else {
-                        let type_name = ob.get_type().name().unwrap();
-                        match type_name {
-                            "datetime" => get_datetime,
-                            "date" => get_date,
-                            "timedelta" => get_timedelta,
-                            "time" => get_time,
-                            "Decimal" => get_decimal,
-                            "range" => get_list,
-                            _ => {
-                                // special branch for np.float as this fails isinstance float
-                                if ob.extract::<f64>().is_ok() {
-                                    return get_float;
-                                }
-
-                                // Can't use pyo3::types::PyDateTime with abi3-py37 feature,
-                                // so need this workaround instead of `isinstance(ob, datetime)`.
-                                let bases = ob
-                                    .get_type()
-                                    .getattr(intern!(py, "__bases__"))
-                                    .unwrap()
-                                    .iter()
-                                    .unwrap();
-                                for base in bases {
-                                    let parent_type =
-                                        base.unwrap().str().unwrap().to_str().unwrap();
-                                    match parent_type {
-                                        "<class 'datetime.datetime'>" => {
-                                            // `datetime.datetime` is a subclass of `datetime.date`,
-                                            // so need to check `datetime.datetime` first
-                                            return get_datetime;
-                                        },
-                                        "<class 'datetime.date'>" => {
-                                            return get_date;
-                                        },
-                                        _ => (),
-                                    }
-                                }
-
-                                get_object
-                            },
-                        }
-                    }
-                },
-            );
-
+            let convert_fn = lut
+                .entry(type_object_ptr)
+                .or_insert_with(|| get_conversion_function(ob, py));
             convert_fn(ob, strict)
         })
     })
