@@ -1,6 +1,5 @@
 pub(crate) mod any_value;
 pub(crate) mod chunked_array;
-
 use std::fmt::{Display, Formatter};
 use std::hash::{Hash, Hasher};
 
@@ -10,17 +9,19 @@ use polars::frame::row::Row;
 use polars::frame::NullStrategy;
 #[cfg(feature = "avro")]
 use polars::io::avro::AvroCompression;
+#[cfg(feature = "cloud")]
+use polars::io::cloud::CloudOptions;
 use polars::series::ops::NullBehavior;
 use polars_core::utils::arrow::array::Array;
 use polars_core::utils::arrow::types::NativeType;
+use polars_core::utils::materialize_dyn_int;
 use polars_lazy::prelude::*;
-#[cfg(feature = "cloud")]
-use polars_rs::io::cloud::CloudOptions;
 use polars_utils::total_ord::{TotalEq, TotalHash};
 use pyo3::basic::CompareOp;
 use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::intern;
 use pyo3::prelude::*;
+use pyo3::pybacked::PyBackedStr;
 use pyo3::types::{PyDict, PyList, PySequence};
 use smartstring::alias::String as SmartString;
 
@@ -138,7 +139,7 @@ fn decimal_to_digits(v: i128, buf: &mut [u128; 3]) -> usize {
 
 impl ToPyObject for Wrap<DataType> {
     fn to_object(&self, py: Python) -> PyObject {
-        let pl = POLARS.as_ref(py);
+        let pl = POLARS.bind(py);
 
         match &self.0 {
             DataType::Int8 => {
@@ -177,7 +178,7 @@ impl ToPyObject for Wrap<DataType> {
                 let class = pl.getattr(intern!(py, "Float32")).unwrap();
                 class.call0().unwrap().into()
             },
-            DataType::Float64 => {
+            DataType::Float64 | DataType::Unknown(UnknownKind::Float) => {
                 let class = pl.getattr(intern!(py, "Float64")).unwrap();
                 class.call0().unwrap().into()
             },
@@ -190,7 +191,7 @@ impl ToPyObject for Wrap<DataType> {
                 let class = pl.getattr(intern!(py, "Boolean")).unwrap();
                 class.call0().unwrap().into()
             },
-            DataType::String => {
+            DataType::String | DataType::Unknown(UnknownKind::Str) => {
                 let class = pl.getattr(intern!(py, "String")).unwrap();
                 class.call0().unwrap().into()
             },
@@ -252,7 +253,7 @@ impl ToPyObject for Wrap<DataType> {
                     let dtype = Wrap(fld.data_type().clone()).to_object(py);
                     field_class.call1((name, dtype)).unwrap()
                 });
-                let fields = PyList::new(py, iter);
+                let fields = PyList::new_bound(py, iter);
                 let struct_class = pl.getattr(intern!(py, "Struct")).unwrap();
                 struct_class.call1((fields,)).unwrap().into()
             },
@@ -260,7 +261,10 @@ impl ToPyObject for Wrap<DataType> {
                 let class = pl.getattr(intern!(py, "Null")).unwrap();
                 class.call0().unwrap().into()
             },
-            DataType::Unknown => {
+            DataType::Unknown(UnknownKind::Int(v)) => {
+                Wrap(materialize_dyn_int(*v).dtype()).to_object(py)
+            },
+            DataType::Unknown(_) => {
                 let class = pl.getattr(intern!(py, "Unknown")).unwrap();
                 class.call0().unwrap().into()
             },
@@ -318,7 +322,7 @@ impl FromPyObject<'_> for Wrap<DataType> {
                     "List" => DataType::List(Box::new(DataType::Null)),
                     "Struct" => DataType::Struct(vec![]),
                     "Null" => DataType::Null,
-                    "Unknown" => DataType::Unknown,
+                    "Unknown" => DataType::Unknown(Default::default()),
                     dt => {
                         return Err(PyTypeError::new_err(format!(
                             "'{dt}' is not a Polars data type",
@@ -354,7 +358,7 @@ impl FromPyObject<'_> for Wrap<DataType> {
             "Float32" => DataType::Float32,
             "Float64" => DataType::Float64,
             "Null" => DataType::Null,
-            "Unknown" => DataType::Unknown,
+            "Unknown" => DataType::Unknown(Default::default()),
             "Duration" => {
                 let time_unit = ob.getattr(intern!(py, "time_unit")).unwrap();
                 let time_unit = time_unit.extract::<Wrap<TimeUnit>>()?.0;
@@ -439,10 +443,10 @@ impl FromPyObject<'_> for Wrap<Schema> {
         Ok(Wrap(
             dict.iter()
                 .map(|(key, val)| {
-                    let key = key.extract::<&str>()?;
+                    let key = key.extract::<PyBackedStr>()?;
                     let val = val.extract::<Wrap<DataType>>()?;
 
-                    Ok(Field::new(key, val.0))
+                    Ok(Field::new(&key, val.0))
                 })
                 .collect::<PyResult<Schema>>()?,
         ))
@@ -457,7 +461,7 @@ pub struct ObjectValue {
 
 impl Hash for ObjectValue {
     fn hash<H: Hasher>(&self, state: &mut H) {
-        let h = Python::with_gil(|py| self.inner.as_ref(py).hash().expect("should be hashable"));
+        let h = Python::with_gil(|py| self.inner.bind(py).hash().expect("should be hashable"));
         state.write_isize(h)
     }
 }
@@ -469,8 +473,8 @@ impl PartialEq for ObjectValue {
         Python::with_gil(|py| {
             match self
                 .inner
-                .as_ref(py)
-                .rich_compare(other.inner.as_ref(py), CompareOp::Eq)
+                .bind(py)
+                .rich_compare(other.inner.bind(py), CompareOp::Eq)
             {
                 Ok(result) => result.is_truthy().unwrap(),
                 Err(_) => false,
@@ -559,7 +563,7 @@ impl<'a, T: NativeType + FromPyObject<'a>> FromPyObject<'a> for Wrap<Vec<T>> {
 #[cfg(feature = "asof_join")]
 impl FromPyObject<'_> for Wrap<AsofStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*(ob.extract::<PyBackedStr>()?) {
             "backward" => AsofStrategy::Backward,
             "forward" => AsofStrategy::Forward,
             "nearest" => AsofStrategy::Nearest,
@@ -575,7 +579,7 @@ impl FromPyObject<'_> for Wrap<AsofStrategy> {
 
 impl FromPyObject<'_> for Wrap<InterpolationMethod> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*(ob.extract::<PyBackedStr>()?) {
             "linear" => InterpolationMethod::Linear,
             "nearest" => InterpolationMethod::Nearest,
             v => {
@@ -591,7 +595,7 @@ impl FromPyObject<'_> for Wrap<InterpolationMethod> {
 #[cfg(feature = "avro")]
 impl FromPyObject<'_> for Wrap<Option<AvroCompression>> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "uncompressed" => None,
             "snappy" => Some(AvroCompression::Snappy),
             "deflate" => Some(AvroCompression::Deflate),
@@ -607,7 +611,7 @@ impl FromPyObject<'_> for Wrap<Option<AvroCompression>> {
 
 impl FromPyObject<'_> for Wrap<CategoricalOrdering> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "physical" => CategoricalOrdering::Physical,
             "lexical" => CategoricalOrdering::Lexical,
             v => {
@@ -622,7 +626,7 @@ impl FromPyObject<'_> for Wrap<CategoricalOrdering> {
 
 impl FromPyObject<'_> for Wrap<StartBy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "window" => StartBy::WindowBound,
             "datapoint" => StartBy::DataPoint,
             "monday" => StartBy::Monday,
@@ -644,7 +648,7 @@ impl FromPyObject<'_> for Wrap<StartBy> {
 
 impl FromPyObject<'_> for Wrap<ClosedWindow> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "left" => ClosedWindow::Left,
             "right" => ClosedWindow::Right,
             "both" => ClosedWindow::Both,
@@ -662,7 +666,7 @@ impl FromPyObject<'_> for Wrap<ClosedWindow> {
 #[cfg(feature = "csv")]
 impl FromPyObject<'_> for Wrap<CsvEncoding> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "utf8" => CsvEncoding::Utf8,
             "utf8-lossy" => CsvEncoding::LossyUtf8,
             v => {
@@ -678,7 +682,7 @@ impl FromPyObject<'_> for Wrap<CsvEncoding> {
 #[cfg(feature = "ipc")]
 impl FromPyObject<'_> for Wrap<Option<IpcCompression>> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "uncompressed" => None,
             "lz4" => Some(IpcCompression::LZ4),
             "zstd" => Some(IpcCompression::ZSTD),
@@ -694,7 +698,7 @@ impl FromPyObject<'_> for Wrap<Option<IpcCompression>> {
 
 impl FromPyObject<'_> for Wrap<JoinType> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "inner" => JoinType::Inner,
             "left" => JoinType::Left,
             "outer" => JoinType::Outer{coalesce: false},
@@ -715,7 +719,7 @@ impl FromPyObject<'_> for Wrap<JoinType> {
 
 impl FromPyObject<'_> for Wrap<Label> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "left" => Label::Left,
             "right" => Label::Right,
             "datapoint" => Label::DataPoint,
@@ -731,7 +735,7 @@ impl FromPyObject<'_> for Wrap<Label> {
 
 impl FromPyObject<'_> for Wrap<ListToStructWidthStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "first_non_null" => ListToStructWidthStrategy::FirstNonNull,
             "max_width" => ListToStructWidthStrategy::MaxWidth,
             v => {
@@ -746,7 +750,7 @@ impl FromPyObject<'_> for Wrap<ListToStructWidthStrategy> {
 
 impl FromPyObject<'_> for Wrap<NonExistent> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "null" => NonExistent::Null,
             "raise" => NonExistent::Raise,
             v => {
@@ -761,7 +765,7 @@ impl FromPyObject<'_> for Wrap<NonExistent> {
 
 impl FromPyObject<'_> for Wrap<NullBehavior> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "drop" => NullBehavior::Drop,
             "ignore" => NullBehavior::Ignore,
             v => {
@@ -776,7 +780,7 @@ impl FromPyObject<'_> for Wrap<NullBehavior> {
 
 impl FromPyObject<'_> for Wrap<NullStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "ignore" => NullStrategy::Ignore,
             "propagate" => NullStrategy::Propagate,
             v => {
@@ -792,7 +796,7 @@ impl FromPyObject<'_> for Wrap<NullStrategy> {
 #[cfg(feature = "parquet")]
 impl FromPyObject<'_> for Wrap<ParallelStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "auto" => ParallelStrategy::Auto,
             "columns" => ParallelStrategy::Columns,
             "row_groups" => ParallelStrategy::RowGroups,
@@ -809,7 +813,7 @@ impl FromPyObject<'_> for Wrap<ParallelStrategy> {
 
 impl FromPyObject<'_> for Wrap<IndexOrder> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "fortran" => IndexOrder::Fortran,
             "c" => IndexOrder::C,
             v => {
@@ -824,7 +828,7 @@ impl FromPyObject<'_> for Wrap<IndexOrder> {
 
 impl FromPyObject<'_> for Wrap<QuantileInterpolOptions> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "lower" => QuantileInterpolOptions::Lower,
             "higher" => QuantileInterpolOptions::Higher,
             "nearest" => QuantileInterpolOptions::Nearest,
@@ -842,7 +846,7 @@ impl FromPyObject<'_> for Wrap<QuantileInterpolOptions> {
 
 impl FromPyObject<'_> for Wrap<RankMethod> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "min" => RankMethod::Min,
             "max" => RankMethod::Max,
             "average" => RankMethod::Average,
@@ -861,7 +865,7 @@ impl FromPyObject<'_> for Wrap<RankMethod> {
 
 impl FromPyObject<'_> for Wrap<Roll> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "raise" => Roll::Raise,
             "forward" => Roll::Forward,
             "backward" => Roll::Backward,
@@ -877,7 +881,7 @@ impl FromPyObject<'_> for Wrap<Roll> {
 
 impl FromPyObject<'_> for Wrap<TimeUnit> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "ns" => TimeUnit::Nanoseconds,
             "us" => TimeUnit::Microseconds,
             "ms" => TimeUnit::Milliseconds,
@@ -893,7 +897,7 @@ impl FromPyObject<'_> for Wrap<TimeUnit> {
 
 impl FromPyObject<'_> for Wrap<UniqueKeepStrategy> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "first" => UniqueKeepStrategy::First,
             "last" => UniqueKeepStrategy::Last,
             "none" => UniqueKeepStrategy::None,
@@ -911,7 +915,7 @@ impl FromPyObject<'_> for Wrap<UniqueKeepStrategy> {
 #[cfg(feature = "ipc")]
 impl FromPyObject<'_> for Wrap<IpcCompression> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "zstd" => IpcCompression::ZSTD,
             "lz4" => IpcCompression::LZ4,
             v => {
@@ -927,7 +931,7 @@ impl FromPyObject<'_> for Wrap<IpcCompression> {
 #[cfg(feature = "search_sorted")]
 impl FromPyObject<'_> for Wrap<SearchSortedSide> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "any" => SearchSortedSide::Any,
             "left" => SearchSortedSide::Left,
             "right" => SearchSortedSide::Right,
@@ -943,7 +947,7 @@ impl FromPyObject<'_> for Wrap<SearchSortedSide> {
 
 impl FromPyObject<'_> for Wrap<ClosedInterval> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "both" => ClosedInterval::Both,
             "left" => ClosedInterval::Left,
             "right" => ClosedInterval::Right,
@@ -960,7 +964,7 @@ impl FromPyObject<'_> for Wrap<ClosedInterval> {
 
 impl FromPyObject<'_> for Wrap<WindowMapping> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "group_to_rows" => WindowMapping::GroupsToRows,
             "join" => WindowMapping::Join,
             "explode" => WindowMapping::Explode,
@@ -976,7 +980,7 @@ impl FromPyObject<'_> for Wrap<WindowMapping> {
 
 impl FromPyObject<'_> for Wrap<JoinValidation> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "1:1" => JoinValidation::OneToOne,
             "1:m" => JoinValidation::OneToMany,
             "m:m" => JoinValidation::ManyToMany,
@@ -994,7 +998,7 @@ impl FromPyObject<'_> for Wrap<JoinValidation> {
 #[cfg(feature = "csv")]
 impl FromPyObject<'_> for Wrap<QuoteStyle> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "always" => QuoteStyle::Always,
             "necessary" => QuoteStyle::Necessary,
             "non_numeric" => QuoteStyle::NonNumeric,
@@ -1018,7 +1022,7 @@ pub(crate) fn parse_cloud_options(uri: &str, kv: Vec<(String, String)>) -> PyRes
 #[cfg(feature = "list_sets")]
 impl FromPyObject<'_> for Wrap<SetOperation> {
     fn extract(ob: &PyAny) -> PyResult<Self> {
-        let parsed = match ob.extract::<&str>()? {
+        let parsed = match &*ob.extract::<PyBackedStr>()? {
             "union" => SetOperation::Union,
             "difference" => SetOperation::Difference,
             "intersection" => SetOperation::Intersection,
