@@ -1,8 +1,8 @@
 #![feature(vec_into_raw_parts)]
 #![allow(clippy::nonstandard_macro_braces)] // Needed because clippy does not understand proc macro of PyO3
 #![allow(clippy::transmute_undefined_repr)]
+#![allow(non_local_definitions)]
 #![allow(clippy::too_many_arguments)] // Python functions can have many arguments due to default arguments
-extern crate polars as polars_rs;
 
 #[cfg(feature = "build_info")]
 #[macro_use]
@@ -28,6 +28,8 @@ mod gil_once_cell;
 mod lazyframe;
 mod lazygroupby;
 mod map;
+#[cfg(debug_assertions)]
+mod memory;
 #[cfg(feature = "object")]
 mod object;
 #[cfg(feature = "object")]
@@ -40,7 +42,7 @@ mod sql;
 mod to_numpy;
 mod utils;
 
-#[cfg(all(target_family = "unix", not(use_mimalloc)))]
+#[cfg(all(target_family = "unix", not(use_mimalloc), not(default_allocator)))]
 use jemallocator::Jemalloc;
 #[cfg(any(not(target_family = "unix"), use_mimalloc))]
 use mimalloc::MiMalloc;
@@ -54,23 +56,43 @@ use crate::conversion::Wrap;
 use crate::dataframe::PyDataFrame;
 use crate::error::{
     CategoricalRemappingWarning, ColumnNotFoundError, ComputeError, DuplicateError,
-    InvalidOperationError, NoDataError, OutOfBoundsError, PolarsBaseError, PolarsBaseWarning,
-    PyPolarsErr, SchemaError, SchemaFieldNotFoundError, StructFieldNotFoundError,
+    InvalidOperationError, MapWithoutReturnDtypeWarning, NoDataError, OutOfBoundsError,
+    PolarsBaseError, PolarsBaseWarning, PyPolarsErr, SchemaError, SchemaFieldNotFoundError,
+    StructFieldNotFoundError,
 };
 use crate::expr::PyExpr;
 use crate::functions::PyStringCacheHolder;
 use crate::lazyframe::{PyInProcessQuery, PyLazyFrame};
 use crate::lazygroupby::PyLazyGroupBy;
+#[cfg(debug_assertions)]
+use crate::memory::TracemallocAllocator;
 use crate::series::PySeries;
 #[cfg(feature = "sql")]
 use crate::sql::PySQLContext;
 
+// On Windows tracemalloc does work. However, we build abi3 wheels, and the
+// relevant C APIs are not part of the limited stable CPython API. As a result,
+// linking breaks on Windows if we use tracemalloc C APIs. So we only use this
+// on Windows for now.
 #[global_allocator]
-#[cfg(all(target_family = "unix", not(use_mimalloc)))]
+#[cfg(all(target_family = "unix", debug_assertions, not(default_allocator)))]
+static ALLOC: TracemallocAllocator<Jemalloc> = TracemallocAllocator::new(Jemalloc);
+
+#[global_allocator]
+#[cfg(all(
+    target_family = "unix",
+    not(use_mimalloc),
+    not(debug_assertions),
+    not(default_allocator)
+))]
 static ALLOC: Jemalloc = Jemalloc;
 
 #[global_allocator]
-#[cfg(any(not(target_family = "unix"), use_mimalloc))]
+#[cfg(all(
+    any(not(target_family = "unix"), use_mimalloc),
+    not(debug_assertions),
+    not(default_allocator)
+))]
 static ALLOC: MiMalloc = MiMalloc;
 
 #[pymodule]
@@ -97,12 +119,12 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::concat_df_horizontal))
         .unwrap();
-    m.add_wrapped(wrap_pyfunction!(functions::eager_int_range))
-        .unwrap();
 
     // Functions - range
     m.add_wrapped(wrap_pyfunction!(functions::int_range))
         .unwrap();
+    m.add_wrapped(wrap_pyfunction!(functions::eager_int_range))
+        .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::int_ranges))
         .unwrap();
     m.add_wrapped(wrap_pyfunction!(functions::date_range))
@@ -118,6 +140,10 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(functions::time_ranges))
         .unwrap();
 
+    // Functions - business
+    m.add_wrapped(wrap_pyfunction!(functions::business_day_count))
+        .unwrap();
+
     // Functions - aggregation
     m.add_wrapped(wrap_pyfunction!(functions::all_horizontal))
         .unwrap();
@@ -204,6 +230,12 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
     #[cfg(feature = "parquet")]
     m.add_wrapped(wrap_pyfunction!(functions::read_parquet_schema))
         .unwrap();
+    #[cfg(feature = "clipboard")]
+    m.add_wrapped(wrap_pyfunction!(functions::read_clipboard_string))
+        .unwrap();
+    #[cfg(feature = "clipboard")]
+    m.add_wrapped(wrap_pyfunction!(functions::write_clipboard_string))
+        .unwrap();
 
     // Functions - meta
     m.add_wrapped(wrap_pyfunction!(functions::get_index_type))
@@ -296,6 +328,11 @@ fn polars(py: Python, m: &PyModule) -> PyResult<()> {
         py.get_type::<CategoricalRemappingWarning>(),
     )
     .unwrap();
+    m.add(
+        "MapWithoutReturnDtypeWarning",
+        py.get_type::<MapWithoutReturnDtypeWarning>(),
+    )
+    .unwrap();
 
     // Build info
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
