@@ -526,8 +526,8 @@ def test_read_database_mocked(
                 read_method="read_database_uri",
                 query="SELECT * FROM test_data",
                 protocol="mysql",
-                errclass=ImportError,
-                errmsg="ADBC mysql driver not detected",
+                errclass=ModuleNotFoundError,
+                errmsg="ADBC 'adbc_driver_mysql.dbapi' driver not detected.",
                 engine="adbc",
             ),
             id="Unavailable adbc driver",
