@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import gzip
 import io
+import os
 import sys
 import textwrap
 import zlib
@@ -2081,3 +2082,18 @@ def test_fsspec_not_available(monkeypatch: pytest.MonkeyPatch) -> None:
         pl.read_csv(
             "s3://foods/cabbage.csv", storage_options={"key": "key", "secret": "secret"}
         )
+
+
+@pytest.mark.write_disk()
+@pytest.mark.skipif(
+    os.environ.get("POLARS_FORCE_ASYNC") == "1" or sys.platform == "win32",
+    reason="only local",
+)
+def test_no_glob(tmpdir: Path) -> None:
+    df = pl.DataFrame({"foo": 1})
+    p = tmpdir / "*.csv"
+    df.write_csv(str(p))
+    p = tmpdir / "*1.csv"
+    df.write_csv(str(p))
+    p = tmpdir / "*.csv"
+    assert_frame_equal(pl.read_csv(str(p), glob=False), df)
