@@ -1,10 +1,13 @@
 from __future__ import annotations
 
 import io
+import os
+import sys
 from datetime import datetime, time, timezone
 from decimal import Decimal
 from typing import TYPE_CHECKING, cast
 
+import fsspec
 import numpy as np
 import pandas as pd
 import pyarrow as pa
@@ -19,6 +22,7 @@ if TYPE_CHECKING:
     from pathlib import Path
 
     from polars.type_aliases import ParquetCompression
+    from tests.unit.conftest import MemoryUsage
 
 
 def test_round_trip(df: pl.DataFrame) -> None:
@@ -679,6 +683,20 @@ def test_read_parquet_binary_file_io(tmp_path: Path) -> None:
     assert_frame_equal(out, df)
 
 
+# https://github.com/pola-rs/polars/issues/15760
+@pytest.mark.write_disk()
+def test_read_parquet_binary_fsspec(tmp_path: Path) -> None:
+    tmp_path.mkdir(exist_ok=True)
+
+    df = pl.DataFrame({"a": [1, 2, 3]})
+    file_path = tmp_path / "test.parquet"
+    df.write_parquet(file_path)
+
+    with fsspec.open(file_path) as f:
+        out = pl.read_parquet(f)
+    assert_frame_equal(out, df)
+
+
 def test_read_parquet_binary_bytes_io() -> None:
     df = pl.DataFrame({"a": [1, 2, 3]})
     f = io.BytesIO()
@@ -775,12 +793,82 @@ def test_parquet_array_dtype() -> None:
 
 
 @pytest.mark.write_disk()
-def test_parquet_array_statistics() -> None:
+def test_parquet_array_statistics(tmp_path: Path) -> None:
+    tmp_path.mkdir(exist_ok=True)
+
     df = pl.DataFrame({"a": [[1, 2, 3], [4, 5, 6], [7, 8, 9]], "b": [1, 2, 3]})
+    file_path = tmp_path / "test.parquet"
+
     df.with_columns(a=pl.col("a").list.to_array(3)).lazy().filter(
         pl.col("a") != [1, 2, 3]
     ).collect()
-    df.with_columns(a=pl.col("a").list.to_array(3)).lazy().sink_parquet("test.parquet")
-    assert pl.scan_parquet("test.parquet").filter(
-        pl.col("a") != [1, 2, 3]
-    ).collect().to_dict(as_series=False) == {"a": [[4, 5, 6], [7, 8, 9]], "b": [2, 3]}
+    df.with_columns(a=pl.col("a").list.to_array(3)).lazy().sink_parquet(file_path)
+
+    result = pl.scan_parquet(file_path).filter(pl.col("a") != [1, 2, 3]).collect()
+    assert result.to_dict(as_series=False) == {"a": [[4, 5, 6], [7, 8, 9]], "b": [2, 3]}
+
+
+@pytest.mark.slow()
+@pytest.mark.write_disk()
+def test_read_parquet_only_loads_selected_columns_15098(
+    memory_usage_without_pyarrow: MemoryUsage, tmp_path: Path
+) -> None:
+    """Only requested columns are loaded by ``read_parquet()``."""
+    tmp_path.mkdir(exist_ok=True)
+
+    # Each column will be about 8MB of RAM
+    series = pl.arange(0, 1_000_000, dtype=pl.Int64, eager=True)
+
+    file_path = tmp_path / "multicolumn.parquet"
+    df = pl.DataFrame(
+        {
+            "a": series,
+            "b": series,
+        }
+    )
+    df.write_parquet(file_path)
+    del df, series
+
+    memory_usage_without_pyarrow.reset_tracking()
+
+    # Only load one column:
+    df = pl.read_parquet([file_path], columns=["b"], rechunk=False)
+    del df
+    # Only one column's worth of memory should be used; 2 columns would be
+    # 16_000_000 at least, but there's some overhead.
+    assert 8_000_000 < memory_usage_without_pyarrow.get_peak() < 13_000_000
+
+
+@pytest.mark.release()
+@pytest.mark.write_disk()
+def test_max_statistic_parquet_writer(tmp_path: Path) -> None:
+    # this hits the maximal page size
+    # so the row group will be split into multiple pages
+    # the page statistics need to be correctly reduced
+    # for this query to make sense
+    n = 150_000
+
+    tmp_path.mkdir(exist_ok=True)
+
+    # int64 is important to hit the page size
+    df = pl.int_range(0, n, eager=True, dtype=pl.Int64).alias("int").to_frame()
+    f = tmp_path / "tmp.parquet"
+    df.write_parquet(f, statistics=True, use_pyarrow=False, row_group_size=n)
+    result = pl.scan_parquet(f).filter(pl.col("int") > n - 3).collect()
+    expected = pl.DataFrame({"int": [149998, 149999]})
+    assert_frame_equal(result, expected)
+
+
+@pytest.mark.write_disk()
+@pytest.mark.skipif(
+    os.environ.get("POLARS_FORCE_ASYNC") == "1" or sys.platform == "win32",
+    reason="only local",
+)
+def test_no_glob(tmpdir: Path) -> None:
+    df = pl.DataFrame({"foo": 1})
+    p = tmpdir / "*.parquet"
+    df.write_parquet(str(p))
+    p = tmpdir / "*1.parquet"
+    df.write_parquet(str(p))
+    p = tmpdir / "*.parquet"
+    assert_frame_equal(pl.scan_parquet(str(p), glob=False).collect(), df)
