@@ -13,7 +13,7 @@ import polars as pl
 import polars.selectors as cs
 from polars.exceptions import NoDataError, ParameterCollisionError
 from polars.io.spreadsheet.functions import _identify_workbook
-from polars.testing import assert_frame_equal
+from polars.testing import assert_frame_equal, assert_series_equal
 
 if TYPE_CHECKING:
     from polars.type_aliases import ExcelSpreadsheetEngine, SchemaDict, SelectorType
@@ -242,6 +242,30 @@ def test_read_excel_basic_datatypes(
         )
         assert_frame_equal(df, df)
 
+    # check some additional overrides
+    # (note: xlsx2csv can't currently convert datetime with trailing '00:00:00' to date)
+    dt_override = {"datetime": pl.Date} if engine != "xlsx2csv" else {}
+    df = pl.read_excel(
+        xls,
+        sheet_id=sheet_id,
+        sheet_name=sheet_name,
+        engine=engine,
+        schema_overrides={"A": pl.Float32, **dt_override},
+    )
+    assert_series_equal(
+        df["A"],
+        pl.Series(name="A", values=[1.0, 2.0, 3.0, 4.0, 5.0], dtype=pl.Float32),
+    )
+    if dt_override:
+        assert_series_equal(
+            df["datetime"],
+            pl.Series(
+                name="datetime",
+                values=[date(2023, 1, x) for x in range(1, 6)],
+                dtype=pl.Date,
+            ),
+        )
+
 
 @pytest.mark.parametrize(
     ("read_spreadsheet", "source", "params"),
@@ -436,6 +460,29 @@ def test_schema_overrides(path_xlsx: Path, path_xlsb: Path, path_ods: Path) -> N
         assert df["test4"].schema[col] == dtype
 
 
+@pytest.mark.parametrize(
+    ("engine", "read_opts_param"),
+    [
+        ("xlsx2csv", "infer_schema_length"),
+        ("calamine", "schema_sample_rows"),
+    ],
+)
+def test_invalid_parameter_combinations(
+    path_xlsx: Path, engine: str, read_opts_param: str
+) -> None:
+    with pytest.raises(
+        ParameterCollisionError,
+        match=f"cannot specify both `infer_schema_length`.*{read_opts_param}",
+    ):
+        pl.read_excel(  # type: ignore[call-overload]
+            path_xlsx,
+            sheet_id=1,
+            engine=engine,
+            read_options={read_opts_param: 512},
+            infer_schema_length=1024,
+        )
+
+
 def test_unsupported_engine() -> None:
     with pytest.raises(NotImplementedError):
         pl.read_excel(None, engine="foo")  # type: ignore[call-overload]
@@ -572,7 +619,7 @@ def test_excel_round_trip(write_params: dict[str, Any]) -> None:
 
     engine: ExcelSpreadsheetEngine
     for engine in ("calamine", "xlsx2csv"):  # type: ignore[assignment]
-        table_params = (
+        read_options = (
             {}
             if write_params.get("include_header", True)
             else (
@@ -594,7 +641,7 @@ def test_excel_round_trip(write_params: dict[str, Any]) -> None:
             xls,
             sheet_name="data",
             engine=engine,
-            read_options=table_params,
+            read_options=read_options,
         )[:3].select(df.columns[:3])
         if engine == "xlsx2csv":
             xldf = xldf.with_columns(pl.col("dtm").str.strptime(pl.Date, fmt_strptime))
