@@ -15,6 +15,10 @@ def test_arr_min_max() -> None:
     assert s.arr.max().to_list() == [2, 4]
     assert s.arr.min().to_list() == [1, 3]
 
+    s_with_null = pl.Series("a", [[None, 2], None, [3, 4]], dtype=pl.Array(pl.Int64, 2))
+    assert s_with_null.arr.max().to_list() == [2, None, 4]
+    assert s_with_null.arr.min().to_list() == [2, None, 3]
+
 
 def test_array_min_max_dtype_12123() -> None:
     df = pl.LazyFrame(
@@ -128,22 +132,40 @@ def test_array_arg_min_max() -> None:
 
 
 def test_array_get() -> None:
-    # test index literal
     s = pl.Series(
         "a",
         [[1, 2, 3, 4], [5, 6, None, None], [7, 8, 9, 10]],
         dtype=pl.Array(pl.Int64, 4),
     )
-    out = s.arr.get(1)
+
+    # Test index literal.
+    out = s.arr.get(1, null_on_oob=False)
     expected = pl.Series("a", [2, 6, 8], dtype=pl.Int64)
     assert_series_equal(out, expected)
 
-    # test index expr
-    out = s.arr.get(pl.Series([1, -2, 4]))
-    expected = pl.Series("a", [2, None, None], dtype=pl.Int64)
+    # Null index literal.
+    out_df = s.to_frame().select(pl.col.a.arr.get(pl.lit(None), null_on_oob=False))
+    expected_df = pl.Series("a", [None, None, None], dtype=pl.Int64).to_frame()
+    assert_frame_equal(out_df, expected_df)
+
+    # Out-of-bounds index literal.
+    with pytest.raises(pl.ComputeError, match="get index is out of bounds"):
+        out = s.arr.get(100, null_on_oob=False)
+
+    # Negative index literal.
+    out = s.arr.get(-2, null_on_oob=False)
+    expected = pl.Series("a", [3, None, 9], dtype=pl.Int64)
     assert_series_equal(out, expected)
 
-    # test logical type
+    # Test index expr.
+    with pytest.raises(pl.ComputeError, match="get index is out of bounds"):
+        out = s.arr.get(pl.Series([1, -2, 100]), null_on_oob=False)
+
+    out = s.arr.get(pl.Series([1, -2, 0]), null_on_oob=False)
+    expected = pl.Series("a", [2, None, 7], dtype=pl.Int64)
+    assert_series_equal(out, expected)
+
+    # Test logical type.
     s = pl.Series(
         "a",
         [
@@ -153,7 +175,53 @@ def test_array_get() -> None:
         ],
         dtype=pl.Array(pl.Date, 2),
     )
-    out = s.arr.get(pl.Series([1, -2, 4]))
+    with pytest.raises(pl.ComputeError, match="get index is out of bounds"):
+        out = s.arr.get(pl.Series([1, -2, 4]), null_on_oob=False)
+
+
+def test_array_get_null_on_oob() -> None:
+    s = pl.Series(
+        "a",
+        [[1, 2, 3, 4], [5, 6, None, None], [7, 8, 9, 10]],
+        dtype=pl.Array(pl.Int64, 4),
+    )
+
+    # Test index literal.
+    out = s.arr.get(1, null_on_oob=True)
+    expected = pl.Series("a", [2, 6, 8], dtype=pl.Int64)
+    assert_series_equal(out, expected)
+
+    # Null index literal.
+    out_df = s.to_frame().select(pl.col.a.arr.get(pl.lit(None), null_on_oob=True))
+    expected_df = pl.Series("a", [None, None, None], dtype=pl.Int64).to_frame()
+    assert_frame_equal(out_df, expected_df)
+
+    # Out-of-bounds index literal.
+    out = s.arr.get(100, null_on_oob=True)
+    expected = pl.Series("a", [None, None, None], dtype=pl.Int64)
+    assert_series_equal(out, expected)
+
+    # Negative index literal.
+    out = s.arr.get(-2, null_on_oob=True)
+    expected = pl.Series("a", [3, None, 9], dtype=pl.Int64)
+    assert_series_equal(out, expected)
+
+    # Test index expr.
+    out = s.arr.get(pl.Series([1, -2, 100]), null_on_oob=True)
+    expected = pl.Series("a", [2, None, None], dtype=pl.Int64)
+    assert_series_equal(out, expected)
+
+    # Test logical type.
+    s = pl.Series(
+        "a",
+        [
+            [datetime.date(1999, 1, 1), datetime.date(2000, 1, 1)],
+            [datetime.date(2001, 10, 1), None],
+            [None, None],
+        ],
+        dtype=pl.Array(pl.Date, 2),
+    )
+    out = s.arr.get(pl.Series([1, -2, 4]), null_on_oob=True)
     expected = pl.Series(
         "a",
         [datetime.date(2000, 1, 1), datetime.date(2001, 10, 1), None],
@@ -366,3 +434,18 @@ def test_array_shift() -> None:
         schema={"lit": pl.Array(pl.Int64, 3), "expr": pl.Array(pl.Int64, 3)},
     )
     assert_frame_equal(out, expected)
+
+
+def test_array_n_unique() -> None:
+    df = pl.DataFrame(
+        {
+            "a": [[1, 1, 2], [3, 3, 3], [None, None, None], None],
+        },
+        schema={"a": pl.Array(pl.Int64, 3)},
+    )
+
+    out = df.select(n_unique=pl.col("a").arr.n_unique())
+    expected = pl.DataFrame(
+        {"n_unique": [2, 1, 1, None]}, schema={"n_unique": pl.UInt32}
+    )
+    assert_frame_equal(out, expected)
