@@ -1684,10 +1684,14 @@ def test_is_finite_is_infinite() -> None:
     assert_series_equal(s.is_infinite(), pl.Series("a", [False, False, True]))
 
 
-def test_is_nan_is_not_nan() -> None:
-    s = pl.Series("a", [1.0, 2.0, 3.0, np.nan])
-    assert_series_equal(s.is_nan(), pl.Series("a", [False, False, False, True]))
-    assert_series_equal(s.is_not_nan(), pl.Series("a", [True, True, True, False]))
+@pytest.mark.parametrize("float_type", [pl.Float32, pl.Float64])
+def test_is_nan_is_not_nan(float_type: pl.PolarsDataType) -> None:
+    s = pl.Series([1.0, np.nan, None], dtype=float_type)
+
+    assert_series_equal(s.is_nan(), pl.Series([False, True, None]))
+    assert_series_equal(s.is_not_nan(), pl.Series([True, False, None]))
+    assert_series_equal(s.fill_nan(2.0), pl.Series([1.0, 2.0, None], dtype=float_type))
+    assert_series_equal(s.drop_nans(), pl.Series([1.0, None], dtype=float_type))
 
 
 def test_dot() -> None:
@@ -2302,3 +2306,34 @@ def test_comp_series_with_str_13123() -> None:
     assert_series_equal(s == "1", pl.Series([True, False, None]))
     assert_series_equal(s.eq_missing("1"), pl.Series([True, False, False]))
     assert_series_equal(s.ne_missing("1"), pl.Series([False, True, True]))
+
+
+@pytest.mark.parametrize(
+    ("data", "single", "multiple", "single_expected", "multiple_expected"),
+    [
+        ([1, 2, 3], 1, [2, 4], 0, [1, 3]),
+        (["a", "b", "c"], "d", ["a", "d"], 3, [0, 3]),
+        ([b"a", b"b", b"c"], b"d", [b"a", b"d"], 3, [0, 3]),
+        (
+            [date(2022, 1, 2), date(2023, 4, 1)],
+            date(2022, 1, 1),
+            [date(1999, 10, 1), date(2024, 1, 1)],
+            0,
+            [0, 2],
+        ),
+        ([1, 2, 3], 1, np.array([2, 4]), 0, [1, 3]),  # test np array.
+    ],
+)
+def test_search_sorted(
+    data: list[Any],
+    single: Any,
+    multiple: list[Any],
+    single_expected: Any,
+    multiple_expected: list[Any],
+) -> None:
+    s = pl.Series(data)
+    single_s = s.search_sorted(single)
+    assert single_s == single_expected
+
+    multiple_s = s.search_sorted(multiple)
+    assert_series_equal(multiple_s, pl.Series(multiple_expected, dtype=pl.UInt32))
