@@ -8,6 +8,7 @@ import pytest
 
 import polars as pl
 from polars.testing import assert_frame_equal
+from polars.testing._constants import PARTITION_LIMIT
 
 if TYPE_CHECKING:
     from pathlib import Path
@@ -100,7 +101,7 @@ def test_streaming_group_by_types() -> None:
             "str_sum": [None],
             "bool_first": [True],
             "bool_last": [False],
-            "bool_mean": [None],
+            "bool_mean": [0.5],
             "bool_sum": [1],
             "date_sum": [date(2074, 1, 1)],
             "date_mean": [date(2022, 1, 1)],
@@ -480,3 +481,34 @@ def test_streaming_groupby_binary_15116() -> None:
         "str": [b"A", b"BB", b"CCCC", b"DDDDDDDD", b"EEEEEEEEEEEEEEEE"],
         "count": [3, 2, 2, 2, 1],
     }
+
+
+def test_streaming_group_by_convert_15380() -> None:
+    assert (
+        pl.DataFrame({"a": [1] * PARTITION_LIMIT}).group_by(b="a").len()["len"].item()
+        == PARTITION_LIMIT
+    )
+
+
+@pytest.mark.parametrize("streaming", [True, False])
+@pytest.mark.parametrize("n_rows", [PARTITION_LIMIT - 1, PARTITION_LIMIT + 3])
+def test_streaming_group_by_boolean_mean_15610(n_rows: int, streaming: bool) -> None:
+    # Also test non-streaming because it sometimes dispatched to streaming agg.
+    expect = pl.DataFrame({"a": [False, True], "c": [0.0, 0.5]})
+
+    n_repeats = n_rows // 3
+    assert n_repeats > 0
+
+    out = (
+        pl.select(
+            a=pl.repeat([True, False, True], n_repeats).explode(),
+            b=pl.repeat([True, False, False], n_repeats).explode(),
+        )
+        .lazy()
+        .group_by("a")
+        .agg(c=pl.mean("b"))
+        .sort("a")
+        .collect(streaming=streaming)
+    )
+
+    assert_frame_equal(out, expect)
