@@ -324,7 +324,7 @@ def test_cse_10401() -> None:
 
     q = df.with_columns(pl.all().fill_null(0).fill_nan(0))
 
-    assert r"""col("clicks").fill_null([0]).alias("__POLARS_CSER""" in q.explain()
+    assert r"""col("clicks").fill_null([0.0]).alias("__POLARS_CSER""" in q.explain()
 
     expected = pl.DataFrame({"clicks": [1.0, 0.0, 0.0]})
     assert_frame_equal(q.collect(comm_subexpr_elim=True), expected)
@@ -512,7 +512,7 @@ def test_no_cse_in_with_context() -> None:
         .with_context(df2.lazy())
         .select(
             pl.col("date_start", "label").gather(
-                pl.col("date_start").search_sorted("timestamp") - 1
+                pl.col("date_start").search_sorted(pl.col("timestamp")) - 1
             ),
         )
     ).collect().to_dict(as_series=False) == {
